

#include "headers.h"
#include "BBUtility.h"

std::string itos(int k)
{
	char buffer[50];

	sprintf(buffer, "%i", k);

	return string(buffer);
}

int random(int lo, int hi)
{
	return int(rand() / 32768.0f * (hi - lo + 1)) + lo;
}


#ifdef _DEBUG
#include "crtdbg.h"
// define the special debugging functions.

std::string myDescription;
int tabs = 0;
int comments = 0;
unsigned int lastCallTime = 0;

class Descriptor
{
public:
	Descriptor(Descriptor *owner = NULL, std::string myname = "");
	~Descriptor();

	void DeleteNextLevel();

	Descriptor *nextLevel;
	Descriptor *parent;

	unsigned int creationTime;
	unsigned int elapsedTime();

	std::string name;
};

Descriptor *myDescriptor = NULL;

Descriptor::Descriptor(Descriptor *owner, std::string myname)
{
	parent = owner;
	nextLevel = NULL;
	
	name = myname;

	creationTime = CL_System::get_time();

}

Descriptor::~Descriptor()
{
	DeleteNextLevel();
}

unsigned int Descriptor::elapsedTime()
{
	return CL_System::get_time() - creationTime;
}

void Descriptor::DeleteNextLevel()
{
	if (nextLevel)
		delete nextLevel;

	nextLevel = NULL;

}

void DB_EndFrame()
{
	
	tabs--;

	DB_AddComment("Ending frame.  Elapsed time: " + itos(myDescriptor->elapsedTime()));
	lastCallTime = CL_System::get_time();

}

void DB_Reset()
{
	if (myDescriptor)
	{
		delete myDescriptor;
		myDescriptor = NULL;
	}

	tabs = 0;
	comments = 0;
	myDescription = "";

	lastCallTime = CL_System::get_time();

}

void DB_DefineNewFrameDescriptor()
{
	

	myDescriptor = new Descriptor;

	_ASSERTE(tabs == 0);
	comments = 0;

	myDescription = "";

	DB_AddComment("Beginning frame...");

	tabs = 1;
	
	lastCallTime = CL_System::get_time();

}

void DB_EnterSubsection(std::string header)
{
	if (!myDescriptor)
		return;

	// make a new descriptor level and set the read/write point to there
	myDescriptor->nextLevel = new Descriptor(myDescriptor, header);
	myDescriptor = myDescriptor->nextLevel;
	

	DB_AddComment("Beginning " + header + ".. (" + itos(CL_System::get_time() - lastCallTime) + ") ..");
	comments = 0;

	tabs++;

	lastCallTime = CL_System::get_time();


}

int DB_ExitSubsection()
{
	if (!myDescriptor)
		return 0;

	tabs--;
	int deltaTime = (signed) myDescriptor->elapsedTime();


	DB_AddComment("Ending " + myDescriptor->name + ".  Elapsed time: " + 
		itos(deltaTime), comments ? false : true );

	myDescriptor = myDescriptor->parent;
	myDescriptor->DeleteNextLevel();

	lastCallTime = CL_System::get_time();

	return deltaTime;
}

void DB_AddComment(std::string comment, bool nocr)
{
	if (nocr)
	{
		myDescription += "\t";
	}
	else
	{
		myDescription += "\n";
	
		for (int i = 0; i < tabs; i++)
			myDescription += "\t";
	}

	myDescription += comment;
	comments++;


}


void DB_Output()
{
	OutputDebugString(myDescription.c_str());
	OutputDebugString("\n\n\n");

}

#endif
