/*****************************************************************************
	Ball: Buster
	Copyright (C) 2004 PatrickAvella

    This file is part of Ball: Buster.

    Ball: Buster is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ball: Buster is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Ball: Buster; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*******************************************************************************
**  Ball: Buster uses the ClanLib sdk, it can be found at http://clanlib.org
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
*******************************************************************************/


#include "ball.h"

CBall::CBall() {
	this->ballx= this->bally= -10;
	this->ballh= this->ballw= 10;
	this->ballvx= 100.0;
	this->ballvy= -400.0;
	this->ballangle = 0;
	this->ballvrot = 30;

	this->ballsticking= true;
	this->stickydifference= 50;
	this->ballstickstart= CL_System::get_time();

	}

// tests to see if two balls collided with one another (this and otherBall)
// and returns true if they did.
bool CBall::collideWith (CBall *otherBall)
{
	// obviously, we don't want to collide with ourself
	if (otherBall == this)
		return false;

	float displacementSqr;					// square of the distance between the two balls
	float displacementX, displacementY;		// two component displacement vector

	// get the displacement vector
	displacementX = otherBall->ballx - ballx;
	displacementY = otherBall->bally - bally;

	displacementSqr = displacementX * displacementX + displacementY * displacementY;

	// don't collide with balls that are stuck if we are moving up
	if (ballsticking && otherBall->ballvy < 0)
		return false;
	else if (ballvy < 0 && otherBall->ballsticking)
		return false;
	
	// check to see if actual collision
	else if (displacementSqr < ballw * ballw && displacementSqr > 1)
	{
		// yep, we collided.  Now we want to calculate to collision.
		// the algorithm is: the balls exchange velocities in the direction of the displacement
		// and velocity perpendicular to that remains unchanged.

		// First we want to normalize the displacement vector.
		displacementSqr = sqrtf(displacementSqr);

		displacementX /= displacementSqr;
		displacementY /= displacementSqr;

		float perpX, perpY;		// vector that is perpendicular to the displacement

		perpX = displacementY;		// simple 2-D algorithm for a perpendicular vector:
		perpY = -displacementX;		// swap the two components and change the sign of one.

		// calculate the components of the velocities in these two directions.
		float thisVParallel, thisVPerp;
		float otherVParallel, otherVPerp;

		// take the scalar product of the normalized displacement vector with the velocity;
		// this gives me the projection of the velocity onto the displacement.
		thisVParallel = displacementX * ballvx + displacementY * ballvy;
		otherVParallel = displacementX * otherBall->ballvx + displacementY * otherBall->ballvy;

		// same for the perpendicular velocity
		thisVPerp = perpX * ballvx + perpY * ballvy;
		otherVPerp = perpX * otherBall->ballvx + perpY * otherBall->ballvy;

		// now swap the parallel components.
		float t;

		t = thisVParallel;
		thisVParallel = otherVParallel;
		otherVParallel = t;

		// reconstruct the actual velocity vector
		ballvx = thisVParallel * displacementX + thisVPerp * perpX;
		ballvy = thisVParallel * displacementY + thisVPerp * perpY;

		otherBall->ballvx = otherVParallel * displacementX + otherVPerp * perpX;
		otherBall->ballvy = otherVParallel * displacementY + otherVPerp * perpY;

		// now make sure the balls aren't touching
		otherBall->ballx = ballx + displacementX * ballw;
		otherBall->bally = bally + displacementY * ballh;

		return true;
	}
	else
	{
		return false;
	}
}


