/*****************************************************************************
	Ball: Buster
	Copyright (C) 2004 PatrickAvella

    This file is part of Ball: Buster.

    Ball: Buster is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ball: Buster is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Ball: Buster; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*******************************************************************************
**  Ball: Buster uses the ClanLib sdk, it can be found at http://clanlib.org
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
*******************************************************************************/

#include "main.h"



testy testyInstance;
// Supposedly i need that ^

testy::testy() 
{
	gamemode= "title";
	// stop mouseMove() from mucking with the paddle during splash()
	attractMode= true;
	paddlex= 350;
	paddlew= 100;
	paddleh= 20;
	paddley= 560;
	paddlerot= 0.0f;
	paddlealpha= 1.0f;
/*	ballx= bally= -10;
	ballh= ballw= 10;
	ballvx= 100.0;
	ballvy= -400.0;
*/

	debugger = false;

	vsync= 1;
	bgy= 0.0f;
	paddleImbueV = 500.0;
	
	fps=1;
	debuginfo= "nothing";
	verticalhit= false;
	stickypaddle = false;
	catchblue = catchred = false;
	catchbluestart = catchredstart = CL_System::get_time();
	fireball = blaster = stageover = transitionout = died = dying = false;
	deathstart= 0;
	fullscreen= true;

	transitionspeed= 0;
	transdelay= 100; transstart= 0;
	transcount= 0;

	playmusic= true;
	bgscroll= true;
	bgspeed= 50.0f;
	level=1;
	thescore= 0;
	freezeScore = false;
	lives=2;
	blocksforitem= 7;
	blocksforpoints= 10;
	blocksdestroyed= 0;

	realPhysics = false;

	hitkey = 0;

//-------------------

	brush= 'r';
	editormode= "";
	mousex= mousey= 0;
	mousedown= false;

	
}

testy::~testy() {
	clearBlocks();
	deleteAllBalls();
	deleteAllPowerUps();
	}

int testy::main(int argc, char **argv) 
{
// uncomment the try/catch blocks and console/cout code to create a more "debug friendly" version.
	
	// Create a console window, yay
//	CL_ConsoleWindow myconsole("A very testy console");
//	myconsole.redirect_stdio();

//	try {
		// initialization, looks like they need to be nested
	unsigned int timer = CL_System::get_time();

	srand(timer);

	CL_SetupCore::init();
	CL_SetupDisplay::init();
	CL_SetupGL::init();
	CL_SetupSound::init();
	CL_SetupVorbis::init();
	CL_SoundOutput output(44100);

	bool fulls = true;

#ifdef _DEBUG
	if (argc > 1)
		if (strcmp(argv[1], "-debug") == 0) debugger = true;
#endif
	if (debugger)
		fulls = false;

	// now it gets fun, set up the display mode
	CL_DisplayWindow mywindow("Ball: Buster", 800, 600, fulls, false);

	displayFormat = &mywindow.get_buffer(0);

	mywindow.hide_cursor();
	// load the images, initiation frame rate counter, and register signals
	img.load();
	snd.load();
	
	mousemotion= CL_Mouse::sig_move().connect(this, &testy::mouseMove);
	mouserelease= CL_Mouse::sig_key_up().connect(this, &testy::mouseRelease);
	mousepress= CL_Mouse::sig_key_down().connect(this, &testy::mousePress);
	keypress = CL_Keyboard::sig_key_down().connect(this, &testy::keyPress);

	loadhighscores();

	myfps= new CL_FramerateCounter();

	//here we go!
	this->run();
	delete myfps;
	img.unload();

	savehighscores();

	// erm, we'll just deinitalize now...
	CL_SetupVorbis::deinit();
	CL_SetupSound::deinit();
	CL_SetupGL::deinit();
	CL_SetupDisplay::deinit();
	CL_SetupCore::deinit();

	
//		}

//	catch (CL_Error error) {
//		std::cout << "something went screwy: " << error.message.c_str() << " \n";
//		myconsole.display_close_message();
//		}


	return 0;
	
}

void testy::run() 
{
	splash();

	while(1) 
	{

		if (gamemode == "level") 
			gamemode= runLevel();

		else if (gamemode == "title") 
			gamemode= runTitle();

		else if (gamemode == "editor") 
			gamemode= runEditor();

		else if (gamemode == "quit") 
			return;

		DB_Reset();
	}

}


void testy::loadLevel() {
	clearBlocks();
    ifstream myfile;
	char charbuf[20];
	itoa(level, charbuf, 10);
	string mystr= charbuf;
	mystr= DATADIR "lvls/" + mystr + ".lvl";
	myfile.open(mystr.c_str());
    if(myfile.bad()) {
		myfile.open(DATADIR "lvls/1.lvl");
		level=1;
		}
	int j= 0;

	invincibleBlocks = 0;
	mystr = "__________________";

	for (j = 0; j < 20; j++) {
		getline(myfile, mystr);
		int i;
		for (i= 0; i < 17; i++){
			if((char)mystr[i] != '_') { 
				addBlock((60 + (40 * i)), 60 + (20 * j), (char)mystr[i]);
				}
			}
		}
	myfile.close();
	level++; // if(level >3 ) level=1;
	if(blocks.size() == 0) {
		level= 1;
		loadLevel();
		}

	}


void testy::draw() 
{
	DB_EnterSubsection("draw");

	CL_Display::clear(CL_Color(0, 0, 0, 255));

// draw the background tile
	img.bgtile->draw(0, -300 + (int)bgy);
	img.bgtile->draw(400, -300 + (int)bgy);
	img.bgtile->draw(0, 0 + (int)bgy);
	img.bgtile->draw(400, 0 + (int)bgy);
	img.bgtile->draw(0, 300 + (int)bgy);
	img.bgtile->draw(400, 300 + (int)bgy);

// draw perimeter
	img.topborder->draw(0,0);
	img.leftborder->draw(0,0);
	img.rightborder->draw(735,0);

// draw blocks and update their animations...
	img.rblock->update();
	img.rblockpart->update();
	img.gblock->update();
	img.gblockpart->update();
	img.bblock->update();
	img.bblockpart->update();
	img.yblock->update();
	img.yblockpart->update();
	img.tblock->update();
	img.tblockpart->update();
	img.wblock->update();
	img.wblockpart->update();

	drawBlocks();

// we draw the flash right on top of the block
	drawFlashes();



// draw paddle, then ball(s)
	DB_EnterSubsection("Drawing Paddle");

	
	img.paddle->update();
	img.smallpaddle->update();
	img.largepaddle->update();

	CL_Sprite *pad;

	if(paddlew == 50) {
		pad = img.smallpaddle;
	}
	else if(paddlew == 100) {
		pad = img.paddle;
	}
	else if(paddlew == 150) {
		pad = img.largepaddle;
	}
	
	pad->set_scale(paddlealpha, paddlealpha);
	pad->set_alpha(paddlealpha);
	pad->set_angle(paddlerot);
	pad->draw((int)paddlex, (int)paddley);
	
	DB_ExitSubsection();


	// draw the power ups
	drawPowerUps();



	DB_EnterSubsection("Drawing balls");

	// draw the balls
	CL_Sprite *ballimg;

	if (fireball)
		ballimg = img.fireball;
	else
		ballimg = img.ball;

	unsigned int j;
	for (j=0; j < balls.size(); j++) 
	{
		ballimg->set_angle(balls[j]->ballangle);
		ballimg->draw((int)balls[j]->ballx, (int)balls[j]->bally);	
	}

	DB_ExitSubsection();



// we'll draw the pieces of block last, so it looks cool....
	drawBlockParts();


	// draw the score bytes last
	drawScoreBytes();


	DB_EnterSubsection("Drawing info");

// draw whatever text

	string message= "Score: " + itos(getScore());

	img.font->set_color(0.0f, 0.0f, 0.0f, 1.0f);	
	img.font->draw(11,585, message);
	img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
	img.font->draw(13,583, message);

#ifdef _DEBUG
	// output debugging messages, if the debugger is attached
	if (debugger)
	{
		img.font->draw(13, 535, "FPS: " + itos(fps));
		img.font->draw(13, 550, "Balls Lost: " + itos(ballslost));
		img.font->draw(13, 565, "powerupcount: " + itos(powerupcount));
	}
#endif

	DB_ExitSubsection();

	// draw extra lives
	int ilives;
	for (ilives= 0; ilives < lives; ilives++) 
	{
		img.ball->draw(775 - (ilives * 13), 588);
	}

	// draw powerups that are in effect:
	powerupLeft = 755;
	powerupTop = 545;
	int count = 0;
	
	if (paddleImbueV > 510)
		drawPowerupInEffect(img.pufastball);
	else if (paddleImbueV < 490)
		drawPowerupInEffect(img.puslowball);
	else
		drawPowerupInEffect(img.puregularspeed);
	
	if (paddlew > 110)
		drawPowerupInEffect(img.pupaddlelarge);
	else if (paddlew < 90)
		drawPowerupInEffect(img.pupaddlesmall);
	else
		drawPowerupInEffect(img.pupaddleregular);

	if (supersticky)
		drawPowerupInEffect(img.pusupersticky);
	else if (stickypaddle)
		drawPowerupInEffect(img.pusticky);
	
	if (blaster)
		drawPowerupInEffect(img.publaster);
	
	if (fireball)
		drawPowerupInEffect(img.pufireball);

	if (catchblue)
	{
		int seconds = ((signed)catchbluestart - (signed)CL_System::get_time())/1000 + 30;
		
		drawPowerupInEffect(img.pucatchblue, seconds);

		if (seconds < 0)
			catchblue = false;

	}
	if (catchred)
	{
		int seconds = ((signed)catchredstart - (signed)CL_System::get_time())/1000 + 30;

		
		if (seconds < 0)
		{
			// well, se used up the whole thing.  gain another 250 points.
			catchred = false;
			
			gainPoints(250, powerupLeft + 20, powerupTop + 20);
			snd.powerup->play();

		}

		drawPowerupInEffect(img.pucatchred, seconds);

	}

	// fade the screen to white if we are transitioning out
	if(transitionout) 
	{
		CL_Rect myrect(0, 0, 800, 600);
		CL_Color mycolor(255, 255, 255, transcount);
		CL_Display::fill_rect(myrect, mycolor);
		cout << ".";
	}

	// display paused message
	if (paused)
	{
		CL_Rect myrect(0, 0, 800, 600);
		CL_Color mycolor(0, 0, 0, 100);
		CL_Display::fill_rect(myrect, mycolor);

		if (CL_System::get_time() % 1000 < 500)
		{
			img.font->set_scale(3.0f, 3.0f);
			img.font->draw(350, 250, "PAUSED");

			img.font->set_scale(1.0f, 1.0f);

		}
	}

	// if we're in attract mode, the drawtitle subroutine still has to be called,
	// so don't flip now.  But otherwise, we need to flip.
	if (!attractMode)
	{
		DB_EnterSubsection("Flipping...");
		CL_Display::flip(vsync);
		DB_ExitSubsection();


/*		DB_EnterSubsection("sleeping...");
		CL_System::sleep(1);
		DB_ExitSubsection();
*/
		
		DB_EnterSubsection("keep_alive...");
		CL_System::keep_alive();
		DB_ExitSubsection();

		
	}

	DB_ExitSubsection();
}

void testy::drawPowerupInEffect(CL_Sprite *image, int seconds)
{
	string time;
	int minutes = seconds / 60;
	seconds %= 60;

	time = itos(minutes) + ":";
	if (seconds < 10)
		time += "0";

	time += itos(seconds);

	image->set_alpha(1.0f);
	image->set_scale(1.0f, 1.0f);
	image->draw(powerupLeft, powerupTop);

	if (seconds > -1)
	{
		int fw = img.font->get_width(time);
		int pos = powerupLeft + (40 - fw) / 2;

		img.font->set_color(0, 0, 0, 1.0f);
		img.font->draw(pos, powerupTop + 2, time);
		img.font->draw(pos, powerupTop + 4, time);
		img.font->draw(pos+1, powerupTop + 3, time);
		img.font->draw(pos-1, powerupTop + 3, time);

		img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
		img.font->draw(pos, powerupTop + 3, time);
	}

	powerupTop -= 45;


}

void testy::initLevel()
{

	stageover= false;
	transitionout= false;
	died=false;
	dying = false;

	transcount= 0;
	paddley= 560;
	paddlerot= 0.0f;
	paddlealpha= 1.0f;
	bgspeed= bgscroll ? 50.0f : 0.0f;

	paused = false;

	loadLevel();
	

	deleteAllBalls();
	deleteAllPowerUps();
	deleteAllBlockParts();
	deleteAllScoreBytes();
	deleteAllFlashes();


	freezeScore = true;
	powerUp(RESET);
	freezeScore = false;

	addBall();
	
	powerupcount = 0;
	ballslost = 0;

}

CL_SoundBuffer_Session testy::playRandomSong()
{

	int randsong= CL_System::get_time() % 6;
	CL_SoundBuffer_Session session;
	if (randsong == 0) session = snd.music1->prepare();
	if (randsong == 1) session = snd.music2->prepare();
	if (randsong == 2) session = snd.music3->prepare();
	if (randsong == 3) session = snd.music4->prepare();
	if (randsong == 4) session = snd.music5->prepare();
	if (randsong == 5) session = snd.music6->prepare();

	session.set_looping(true);
	if (playmusic) session.play();

	return session;

}

string testy::runLevel() 
{

	initLevel();

	CL_SoundBuffer_Session session = playRandomSong();
	
	unsigned int start = CL_System::get_time();
	unsigned int end;

	unsigned int lastframetime = CL_System::get_time();
	unsigned int frametime = CL_System::get_time();

	while(!stageover) 
	{
		lastframetime = frametime;
		frametime = CL_System::get_time();

		if (frametime - lastframetime > 50)
		{
#ifdef _DEBUG
//			DB_Output();
//			OutputDebugString(string("Warning... frame time is: " + itos(frametime-lastframetime) + "\n").c_str());
#endif
		}


		DB_DefineNewFrameDescriptor();

		if (paused && session.is_playing())
			session.stop();
		else if (!paused && session.is_playing() == false && playmusic && !transitionout)
			session.play();
		

		fps= myfps->get_fps();

		
		if (!paused)
		{
			
			updateLevel();

		}


		draw();

		
		DB_EnterSubsection("End of level checks");
// see if the level is over...
		if(blocks.size() <= invincibleBlocks && !transitionout && !stageover) 
		{ 
			end = CL_System::get_time();
			transitionout= true;
			if(session.is_playing()) session.stop();
			if (playmusic) snd.speedup->play();
		}

		

// see if the player is dead. If so, take life and give ball, else, game over.
		if(balls.size() == 0 && !transitionout) {
			if(paddlealpha == 1.0f) 
			{
				snd.die->play();
				dying= true;
				paddley -= 100 / fps;
	//			died=false;;
			}
			if(20 + deathstart < CL_System::get_time()) {
				paddlealpha -= 0.02f;
				paddlerot += 20.0f;
				paddley -= 100 / fps;
				deathstart= CL_System::get_time();
				}
			if(paddlealpha < 0.0f) {
				dying=false;
				died=false;


				freezeScore = true;
				powerUp(RESET);
				powerupcount --;
				freezeScore = false;
				

				if (lives == 0) { stageover= true; died= true; }
				if (lives > 0) {
					addBall();
					if(!transitionout) lives--;
					paddley= 560;
					paddlerot= 0.0f;
					paddlealpha= 1.0f;
					}
				}
			}

// if escape is pressed, the boss is coming!
		if (CL_Keyboard::get_keycode(CL_KEY_ESCAPE)) 
			return "title"; 
		if (CL_Keyboard::get_keycode(CL_KEY_END))
		{
			if(session.is_playing()) session.stop();
			return "title";
		}
		DB_ExitSubsection();


		DB_EndFrame();

// end loop......
	}

	transitionout= false;
	bgspeed= bgscroll ? 50.0f : 0.0f;

	if(session.is_playing()) session.stop();
	if(!died) 
	{
		endLevel(end - start);

		return "level";
	}

	endgame();

	return "title";

}

CL_Color HSVtoRGB(float H, float S, float V) 
{
	float R, G, B;

	if (S == 0.0) // color is on black-and-white center line 
	{	R = V;   // achromatic: shades of gray 
		G = V;    // supposedly invalid for h=0 but who cares 
		B = V; 
	}
	else		// -- chromatic color 
	{
		if (H >= 360.0) // -- 360 degrees same as 0 degrees 
			H -= 360.0;
		
		float q, p, t, f;
		int i;

		H = H/60.0f;   // h is now in [0,6) 
		i = int(H);  // largest integer <= h 
		f = H-i;         //- fractional part of h 

		p=V*(1.0f - S);
		q=V*(1.0f - (S * f)); 
		t=V*(1.0f - (S * (1.0f-f))); 

		switch(i)
		{ 
		case 0: 
			R = V; 
			G = t; 
			B = p; 
			break;

		case 1: 
			R = q; 
			G = V; 
			B = p; 
			break;

		case 2: 
			R = p; 
			G = V; 
			B = t; 
			break;

		case 3: 
			R = p; 
			G = q; 
			B = V; 
			break;

		case 4: 
			R = t; 
			G = p; 
			B = V; 
			break;

		case 5: 
			R = V; 
			G = p; 
			B = q; 
			break;
		}
	}
	return CL_Color((unsigned int)(R*255), (unsigned int)(G*255), (unsigned int)(B*255), 255) ;
}

void testy::endgame()
{
	bool addhighscore = false;
	int addposition = 0;

	
	// test to see if we have a new high score
	for (unsigned int i = 0; i < highscores.size(); i++)
	{
		if (getScore() > highscores[i]->score)
		{
			addhighscore = true;
			addposition = i;

			break;
		}
	}

	// also add the high score if the list has less than 10 positions.
	if (highscores.size() < 10 && addhighscore == false)
	{
		addhighscore = true;
		addposition = -1;
	}

	if (getScore() == 0) 
		addhighscore = false;


	if (addhighscore)
	{
		// new high score!
		CHighscore *ns = new CHighscore();

		ns->score = getScore();

		if (addposition > -1)
			highscores.insert(highscores.begin() + addposition, ns);
		else
		{
			addposition = highscores.size();
			highscores.push_back(ns);
		}

		bool hitenter = false;
		unsigned int start = CL_System::get_time();

		while (highscores.size() > 10)
		{
			CHighscore *t = highscores[10];
			
			delete t;

			highscores.erase(highscores.end()-1);
		}

		while(!hitenter) 
		{
			unsigned int time = CL_System::get_time() - start;

			CL_Display::clear(CL_Color(255, 255, 255, 255));


			img.font->set_color(CL_Color(0,0,0,255));
			img.font->set_scale(2.0f, 2.0f);

			img.font->draw(200, 50, string("New High Score!"));
			
			for (unsigned int j = 0; j < highscores.size(); j++)
			{
				int drawy = 100 + 30 * j;

				img.font->set_color(CL_Color(0,0,0,255));

				if (addposition == j)
				{
					img.font->set_color(HSVtoRGB((time % 3600)/10.0f, 0.7f, 0.7f));
				}
			
				img.font->draw(200, drawy, itos(j+1) + ".");
				img.font->draw(230, drawy, highscores[j]->name);
				img.font->draw(550, drawy, itos(highscores[j]->score));

			}

			//img.font->set_scale(1.0f, 1.0f);

			int key = getkeypress();

			if (key)
			{
				if (key == CL_KEY_BACKSPACE && ns->name.size() > 0)
				{
					ns->name.erase(ns->name.end()-1);
				}
				else if (key == CL_KEY_RETURN && ns->name.size() > 0)
				{
					hitenter = true;
				}
				else if (ns->name.size() < 12)
				{
					if (key >= CL_KEY_A && key <= CL_KEY_Z ||
						key >= CL_KEY_0 && key <= CL_KEY_9)
					{
						if ((ns->name.size() + hitstring.size()) <= 12)
							ns->name += hitstring;
					}
					else if (key == CL_KEY_SPACE)
					{
						ns->name += ' ';
					}
				}

			}


			CL_Display::flip(vsync);
			CL_System::sleep(0);
			CL_System::keep_alive();
		}

		img.font->set_scale(1.0f, 1.0f);
	
	
		savehighscores();
	
	}
}

void testy::endLevel(unsigned int levelTime)
{
	unsigned int start= CL_System::get_time();
	unsigned int end = start + 10000;
	unsigned int median;
	float countRate = 1.5;  //r means the numbers count slower
	int bonus = 500;		// basic level bonus for completing a level
	int extraBonus = 0;
	int timeBonus;
	int newscore;
	int time = levelTime / 1000;
	int minutes, seconds;
	int lineCount;
	int lineLimit;
	int tab = 400;
	int totalBonus;

	string timeString;

	seconds = time % 60;
	minutes = time / 60;

	timeString = itos(minutes) + ":";

	if (seconds < 10) timeString += "0";
	timeString += itos(seconds);



	timeBonus = (4 - minutes) * 60 + 60 - seconds;
	if (timeBonus < 0) timeBonus = 0;

	extraBonus = 0;

	/*if (perfectgame)
		extraBonus += 1500;*/
	if (ballslost == 0)
		extraBonus += 500;
	if (powerupcount == 0)
		extraBonus += 1500;

	totalBonus = bonus + timeBonus + extraBonus;
	newscore = getScore() + totalBonus;

	start = CL_System::get_time();
	end = start + 10000;

	while(CL_System::get_time() < end) 
	{
		median = CL_System::get_time() - start;

		CL_Display::clear(CL_Color(255, 255, 255, 255));

	
		lineCount = 0;
		lineLimit = 0;


		img.font->set_color(CL_Color(0,0,0,255));
		img.font->set_scale(2.0f, 2.0f);

		img.font->draw(200, 50, string("Completed level ") + itos(level-1));
		
		//img.font->set_scale(1.0f, 1.0f);

		if (median > 1000)
		{
			img.font->draw(200, 100, string("Score:"));
			img.font->draw(tab, 100, itos(getScore()));
		}

		if (median > 1500)
		{
			img.font->draw(200, 150, string("Level Bonus:"));
			img.font->draw(tab, 150, itos(min(bonus, (int)((median-1500) / countRate))));
		}

		if (median > 2000)
		{
			img.font->draw(200, 200, string("Time: "));
			img.font->draw(tab, 200, timeString);
		}

		if (median > 2500)
		{
			img.font->draw(200, 230, string("Time Bonus: "));
			img.font->draw(tab, 230, itos(min(timeBonus, (int)((median-2500) / countRate))));
		}

		if (median > 3000)
		{
			lineLimit = (median - 3000) / 500 + 1;
		}




		if (ballslost == 0 && lineCount < lineLimit)
		{
			img.font->draw(200, 280 + lineCount * 50, "No balls lost");
			img.font->draw(tab, 280 + lineCount * 50, "500");

			lineCount++;
		}

		/// show other bonuses.
		if (powerupcount == 0 && lineCount < lineLimit)
		{
			img.font->draw(200, 280 + lineCount * 50, "No powerups");
			img.font->draw(tab, 280 + lineCount * 50, "1500");

			lineCount++;
		}
/*
		if (perfectgame && lineCount < lineLimit)
		{
			// perfect game means no extra balls used, did not get blaster powerup
			// and no lives lost
			img.font->draw(200, 280 + lineCount * 50, "Perfect Game");
			img.font->draw(tab, 280 + lineCount * 50, "1500");

			lineCount++;
		}
*/

		/// display the final score
		if (lineCount < lineLimit)
		{
			int medianstart = 3000 + (lineCount) * 500;

			img.font->draw(200, 280 + lineCount * 50, "Final Score:");
			img.font->draw(tab, 280 + lineCount * 50, itos(min(newscore, (int)(getScore() + (median - medianstart) / countRate))));

			lineCount++;
		}


		CL_Display::flip(vsync);
		CL_System::sleep(0);
		CL_System::keep_alive();
	}

	img.font->set_scale(1.0f, 1.0f);

	gainPoints(totalBonus);

}


void testy::mouseMove(const CL_InputEvent &mouse)
{
	
 	lastMouseMove = CL_System::get_time();
	hideTitle = false;
	
	if (!attractMode && !paused)
	{
		paddleVelocity = (float(mouse.mouse_pos.x) - paddlex) * fps;

		paddlex = (float)mouse.mouse_pos.x;		
		
		paddleCheckEdge();
		
//		CL_Mouse::set_position((int)paddlex, mouse.mouse_pos.y);

	}

	mousex= mouse.mouse_pos.x;
	mousey= mouse.mouse_pos.y;


}

bool testy::paddleCheckEdge()
{
	if (paddlex < 60) { paddlex = 60; return true; }
	if (paddlex > 740 - paddlew) { paddlex = 740 - paddlew; return true; }

	return false;
}

void testy::mousePress(const CL_InputEvent &key) 
{
	if (gamemode == "level") 
	{
		unsigned int i;
		for (i= 0; i < balls.size(); i++) 
		{
			if(balls[i]->ballsticking) balls[i]->ballsticking=false;
		}
		if(blaster && !dying)
		{
			addBall(int(paddlew * 0.5f + random(-4, 1)));
		}
		return;
	}
	if (gamemode == "editor") 
	{
		mousedown= true; //handleEditorClicks();
	}

	hideTitle = false;
	lastMouseMove = CL_System::get_time();

}

void testy::mouseRelease(const CL_InputEvent &key) {
	if (gamemode == "level") {

		return;
		}

	if (gamemode == "editor") {
		mousedown= false; //handleEditorClicks();
		if(mousex > 13-20 && mousex < 60 && mousey > 523 && mousey < 538) {
			editormode= "titlescreen";
			}
		if(mousex > 13-20 && mousex < 60 && mousey > 538 && mousey < 553) {
			saveEditor();
			}
		if(mousex > 13-20 && mousex < 60 && mousey > 553 && mousey < 568) {
			level++;
			editormode= "reload";
			}
		if(mousex > 13-20 && mousex < 60 && mousey > 568 && mousey < 583) {
			if(level > 1) level--;
			editormode= "reload";
			}

		}
	if (gamemode == "title") 
	{
		if (hideTitle)
		{
			hideTitle = false;
			lastMouseMove = CL_System::get_time();

			return;
		}
//		mousedown= false; //handleEditorClicks();
		if(mousex > 100-20 && mousex < 500 && mousey > 100 && mousey < 130) {
			titlemode= "startgame";
			}
		if(mousex > 100-20 && mousex < 500 && mousey > 130 && mousey < 160) {
			titlemode= "leveleditor";
			}
		if(mousex > 100-20 && mousex < 500 && mousey > 160 && mousey < 190) {
			titlemode= "quit";
			}
		if(mousex > 100-20 && mousex < 500 && mousey > 190 && mousey < 220) {
			if(fullscreen) { 
				CL_Display::set_windowed(); fullscreen= false;
				CL_Display::set_size(800, 600);
				return;
				}
			if(!fullscreen) { 
				CL_Display::set_fullscreen(800, 600, 32); fullscreen= true; 
				return;
				}
			}
		if(mousex > 100-20 && mousex < 500 && mousey > 220 && mousey < 250) {
			bgscroll= bgscroll ? false : true;
			bgspeed= bgscroll ? 50.0f : 0.0f;
			}
		if(mousex > 100-20 && mousex < 500 && mousey > 250 && mousey < 280) {
			vsync= vsync == 1 ? 0 : 1;
			}
		if(mousex > 100-20 && mousex < 500 && mousey > 280 && mousey < 320) {
			playmusic= playmusic ? false : true;
			}

		}
	}

float testy::PE(CBall *myball)
{
	float g = 150;

	return	2 * g * (myball->bally-600);
}

void testy::SetGravityVelocity (CBall *myball)
{
	if (realPhysics == false)
	{
		float totalv = sqrtf(myball->ballvx * myball->ballvx + myball->ballvy * myball->ballvy);
		myball->maxv = paddleImbueV;

		if (totalv > 0 && abs(int(totalv - myball->maxv)) > 0.1)
		{
			float ratio = myball->maxv / totalv;

			myball->ballvx *= ratio;
			myball->ballvy *= ratio;
		}

		return;
	}

	if (myball->ballsticking == true) return;
	if (myball->bally < 0) return;

	if (myball->ballvx > myball->maxv)
		myball->ballvx = myball->maxv * 0.999f;

	float vyBottom = sqrtf(myball->maxv*myball->maxv - myball->ballvx * myball->ballvx);
	float vysquared;

	static int count = 0;


	vysquared = vyBottom * vyBottom + PE(myball);
	

	if (vysquared > 0)
	{
		if (myball->ballvy > 0)
			myball->ballvy = sqrtf(vysquared);
		else 
			myball->ballvy = -sqrtf(vysquared);

		count = 0;

	}
	else if (vysquared < 0)			// we've reached the vertex of the curve
	{
		myball->ballvy = 1;
		myball->maxv = sqrtf(myball->ballvx * myball->ballvx - PE(myball));

		count ++;

		if (count > 20) 
			myball->maxv = paddleImbueV;

		/*
		if (vysquared < -100000)
		{
			// we are in the territory of ball without enough energy.
			//myball->maxv = paddleImbueV;
			return;
		}
		//if (myball->ballvy > 0)
		// of course, this only happens at the top of the curve.  so we want a positive (downward) velocity
			myball->ballvy = sqrt(-vysquared);
		/*else 
			myball->ballvy = sqrt(-vysquared);*/
	}
	else
	{
		myball->ballvy = myball->maxv;
	}

}

void testy::updateLevel() 
{
	DB_EnterSubsection("updateLevel()");

	if ( fps > 0 ) 
	{
		unsigned int j;
		int lowestball = -1;
		bool badBall = false;
		bool chasepowerup = false;

		CBall *myball;

		DB_EnterSubsection(string("main ball testing loop"));

		for(j=0; j < balls.size(); j++) 
		{
			badBall = false;

			myball = balls[j];

			myball->ballangle += myball->ballvrot / (float)fps;
			if (myball->ballangle > 360)
				myball->ballangle -= 360.0f;

			// doing collision checks for bricks first to determine where the ball hit it.
			if (myball->ballsticking == false)
			{
				myball->ballx+= myball->ballvx / (float) fps;
				unsigned int i;

				for(i=0; i < blocks.size(); i++) 
				{
					if(blocks[i]->collision(myball->ballx, myball->bally, myball->ballw, myball->ballh)) 
					{
						if(!fireball || (blocks[i]->type == 's' || blocks[i]->type == 'i'))
						{
							myball->ballvx= -myball->ballvx;
							myball->ballx += myball->ballvx / (float) fps;
						}
						verticalhit= false;
						
						if (!transitionout) 
							hitBlock(i, myball);

						i = blocks.size();
					}
				}

				// now the y axis.....
				myball->bally+= myball->ballvy / (float) fps;		
				
				SetGravityVelocity(myball);

				i=0;
				for(i=0; i < blocks.size(); i++) 
				{
					if(blocks[i]->collision(myball->ballx, myball->bally, myball->ballw, myball->ballh)) 
					{
						if(!fireball || (blocks[i]->type == 's' || blocks[i]->type == 'i')) 
						{
							myball->ballvy= -myball->ballvy;
							myball->bally += myball->ballvy / (float) fps;
						}
						verticalhit= true;

						if (!transitionout)
							hitBlock(i, myball);

						i = blocks.size();
					}
				}
			}

			// check for ball collisions
			for(unsigned int k = 0; k < balls.size(); k++) 
			{
				if (k != j)
				{
					if (myball->collideWith(balls[k]))
					{
						snd.bounce->play();
					}
				}
			}

			// check the perimeter
			if(myball->ballx < 60) 
			{ 
				myball->ballvx = -myball->ballvx; 
				myball->ballx=60; 
			}
			if(myball->ballx+myball->ballw > 740)
			{
				myball->ballvx = -myball->ballvx; 
				myball->ballx= 740 - myball->ballw; 
			}
			if(myball->bally < 10) 
			{
				if (myball->ballvy < 0) myball->ballvy = -myball->ballvy;

				myball->bally=10; 
			}
			if(myball->bally+myball->ballh > 600) 
			{ 
				badBall = true; 
				snd.ballfall->play(); 
			}

			// check for attract mode
			// don't care about bad balls, and we don't care about balls that are sticking
			// so the paddle should worry about balls that are falling, but if there are none, than just
			// balls that are low
			if (attractMode)
			{
				if (myball->bally < paddley+paddleh && !badBall && !myball->ballsticking && 
					(myball->ballvy > 0 && myball->bally > 300))
				{
					if (lowestball == -1)
						lowestball = j;
					else if ((600 - myball->bally) / myball->ballvy <
						(600 - balls[lowestball]->bally) / balls[lowestball]->ballvy
						&& myball->ballvy > 0)
						lowestball = j;
				}
			}

			// check for ball acceleration, if super magnet is in use
			if (supersticky)
			{
			/*	if (myball->bally > 400 && 
					abs(int(myball->ballx + myball->ballw/2 - (paddlex + paddlew / 2))) < 100)*/
				if (myball->ballvy > 0 || myball->bally > paddley)
				{
					float distx = (paddlex + paddlew / 2 - (myball->ballx + myball->ballw/2));
					float disty = (paddley - myball->bally);
					float denom = powf((distx * distx + disty * disty) / 2500, 1.5);

					if (denom < .001) 
						denom = .001f;

					myball->ballvx += distx / denom * 250 / fps;
					myball->ballvy += disty / denom * 250 / fps;

					// adjust the angle but keep the speed the same.  not realistic, but realism
					// makes the balls accelerate hella fast.
					if (!realPhysics)
					{
						float angle = atan2f(myball->ballvy, myball->ballvx);

						myball->ballvx = paddleImbueV * cosf(angle);
						myball->ballvy = paddleImbueV * sinf(angle);
					}

					//myball->ballvx += 1/(paddlex + paddlew / 2 - myball->ballx - myball->ballw) * 500 / fps;
				}
			}

 
			// check the paddle
			bool hit_paddle= true;
			if(myball->ballx+myball->ballw < paddlex) hit_paddle= false;
			if(myball->ballx > paddlex+paddlew) hit_paddle= false;
			if(myball->bally+myball->ballh < paddley) hit_paddle= false;
			if(myball->bally > paddley+paddleh) hit_paddle= false;

			if (hit_paddle)
			{
				// the difference between the centers divided by paddle length * 2 gives number between -1 and 1... kinda
				float mydif= ( (myball->ballx+ (myball->ballw / 2)) - (paddlex+ (paddlew / 2)) );
				
				if(mydif != 0.0f) mydif /= paddlew;
				mydif *= 2.0f;

				if (mydif > 0.97f) mydif= 0.97f;
				if (mydif < -0.97f) mydif= -0.97f;

				// get the radian from asin();
				mydif= asinf(mydif);

				// apply new velocity....
				myball->ballvy = paddleImbueV * cosf(mydif);
				myball->ballvx = paddleImbueV * sinf(mydif);
				myball->maxv = paddleImbueV;

				if(myball->ballvy > 0.0) myball->ballvy = -myball->ballvy;
				// if you have the stick power up.....
				if (stickypaddle && myball->ballsticking == false ) 
				{
					myball->ballsticking= true;
					myball->stickydifference= myball->ballx - paddlex;
					myball->ballstickstart= CL_System::get_time();
				}
			}

				// if the ball is sticking to the paddle....
			if(myball->ballsticking) 
			{
				
				myball->ballx = paddlex + myball->stickydifference;
				myball->bally = paddley - myball->ballh;
				
				if(unsigned(myball->ballstickstart + 4000) < CL_System::get_time()) 
				{
					myball->ballsticking= false;
				
				}

				if (attractMode && powerups.size() == 0)
				{
					if (rand()/32768.0f < 0.1)
						myball->ballsticking = false;
				}
			}



			if (badBall)
			{
				deleteBall(j);

				if (!transitionout)
				{
					ballslost ++;
				}

			}

		}

		DB_ExitSubsection();

	// update the block fragments and the brick flashes
		updateBlockParts();
		updateFlashes();
		updatePowerUps();
		updateScoreBytes();


	// update bg scroll for fun... tied in with the transitioning 'system' ....
		if (transitionout)  
		{
			paddley -= 100 / (float)fps;
			if(transstart + transdelay < CL_System::get_time()) 
			{
				bgspeed+= 20;
				transcount += 5;
				transstart= CL_System::get_time();
			}
			if (transcount >= 255) 
			{

				stageover= true;
				paddley= 560;
				snd.ching->play();

				transcount = 255;
			}
		}

		bgy += bgspeed / (float)fps;
		if(bgy >= 300.0f) bgy = bgy - 300.0f;

		/// update for attract mode
		// first check to see if we want to chase a powerup instead
		if (attractMode && lowestball == -1)
		{
			chasepowerup = true;

			for (unsigned int i = 0; i < powerups.size(); i++)
			{
				if (lowestball == -1)
					lowestball = i;
				else if (powerups[i]->y > powerups[lowestball]->y)
					lowestball = i;

			}

		}

		if (attractMode && lowestball > -1 && !chasepowerup)
		{
			// the idea is to create a computer player that plays like a human.  
			// the goal:  keep the paddle under the ball.
			// force the computer to accelerate the paddle instead of just using
			// something lame like paddlex = myball->ballx - 10
			int diff = int(balls[lowestball]->ballx - (paddlex + paddlew/2));
			int vdiff = int(balls[lowestball]->ballvx - attractvelocity);
			int sign = 0;

			// check to see if the ball is left or right of the paddle
			if (diff < -paddlew/3) sign = -1;
			if (diff > paddlew/3) sign = 1;

			// if the ball is outside the paddle region, we want to accelerate towards it.
			// the further the ball is the faster we want to accelerate
			attractvelocity += sign * 4 * (abs(diff) + 10);

			// otherwise, if the ball is above the paddle, but there's a large discrepancy
			// between the velocities of the ball and the paddle, we should accelerate 
			// the paddle in the direction the ball is going.
			// but make sure that we don't slow down if they are going towards each other
			/*if (balls[lowestball]->ballvx * attractvelocity < 0 && 
				abs(int(diff + vdiff * .01) < abs(int(diff))) && sign == 0) // && abs(vdiff) > 15*/
			if (balls[lowestball]->ballvx * attractvelocity > 0)
			{  
				if (vdiff > 0)
					attractvelocity += max(vdiff, 10);
				else
					attractvelocity += min(vdiff, -10);
			}

			if (vdiff * attractvelocity > 0 && abs(int(diff)) < paddlew)
			{ 
				if (vdiff > 0)
					attractvelocity += max(vdiff, 10);
				else
					attractvelocity += min(vdiff, -10);
			}

		}
		else if (attractMode && lowestball > -1 && chasepowerup)
		{
			int diff = int((powerups[lowestball]->x + powerups[lowestball]->w/2) - (paddlex + paddlew/2));
			int sign = 0;

			// check to see if the ball is left or right of the paddle
			if (diff < -paddlew/3) sign = -1;
			if (diff > paddlew/3) sign = 1;

			// if the powerup is outside the paddle region, we want to accelerate towards it.
			// the further the powerup is the faster we want to accelerate
			if (abs(attractvelocity) < abs(diff) * 3 || attractvelocity * diff < 0)
				attractvelocity += sign * (abs(diff) + 10);

			if (sign == 0)
				attractvelocity = int(attractvelocity * 0.9f);


		}
		else if (attractMode && lowestball == -1)
		{
			attractvelocity = int(attractvelocity * 0.93f);
		}
 
		if (attractMode)
		{
			if (attractvelocity > 1300) 
				attractvelocity = 1300;
			else if (attractvelocity < -1300)
				attractvelocity = -1300;

			// update the paddle position
			paddlex += attractvelocity / fps;


			// pay attention to the borders
			if (paddleCheckEdge())
				attractvelocity = 0;
			

			// hey, if we've got the blaster, use it.
			if (blaster && rand()/32768.0f < 0.05f)
			{
				addBall();
			}

			paddleVelocity = (float)attractvelocity;
		}
		
	}

	DB_ExitSubsection();
}



void testy::addBlock(int myx, int myy, char color) {

	CBlock *pblock= new CBlock;
	pblock->color= color;
	pblock->setCoords(float(myx), float(myy));
	//pblock->block= img.rblock;
	//pblock->str= 3;
	//pblock->blockpart= img.rblockpart;
	
	if(color == 'r') 
	{
		pblock->block= img.rblock;
		pblock->setStr(3);
		pblock->blockpart= img.rblockpart;
		
	}
	else if(color == 'g') 
	{
		
		pblock->block= img.gblock;
		pblock->setStr(1);
		pblock->blockpart= img.gblockpart;

	}
	else if(color == 'b') 
	{
		
		pblock->block= img.bblock;
		pblock->setStr(2);
		pblock->blockpart= img.bblockpart;
		
	}
	else if(color == 'y') 
	{
		
		pblock->block= img.yblock;
		pblock->setStr(2);
		pblock->blockpart= img.yblockpart;
		
	}
	else if(color == 't') 
	{
		
		pblock->block= img.tblock;
		pblock->setStr(3);
		pblock->blockpart= img.tblockpart;
		
	}
	else if(color == 'w') 
	{
		
		pblock->block= img.wblock;
		pblock->setStr(1);
		pblock->blockpart= img.wblockpart;
		
	}
	else if(color == 's') 
	{

		pblock->block= img.sblock;
		pblock->setStr(50);
		pblock->blockpart= img.sblockpart;
		pblock->type= 'i';
		

		invincibleBlocks++;
		
	}
	else if(color == 'c') 
	{

		pblock->block= img.cblock;
		pblock->setStr(1);
		pblock->blockpart= img.cblockpart;
		pblock->type= 's';
		
	}
	else
	{
		char temp[2];
		temp[0] = color;
		temp[1] = 0;

		fprintf(stderr, "Bad block type: '%s'.\n", temp);

		return;
	}

	blocks.push_back(pblock);


}

void testy::clearBlocks() {

	if (blocks.size() > 0) {
		unsigned int i;
		for (i= 0; i < blocks.size(); i++) {
			delete blocks[i];
			}
		blocks.clear();
		}
	}

void testy::drawBlocks() 
{
	DB_EnterSubsection("drawBlocks");

	if (blocks.size() > 0) 
	{
		unsigned int i;
		float crack;

		for(i=0; i < blocks.size(); i++) 
		{
			blocks[i]->block->draw((int)blocks[i]->getx(), (int)blocks[i]->gety());
	
			crack = blocks[i]->crackPercentage();

			if (crack > 0.001)
			{
				img.crack->set_alpha(crack);
				img.crack->set_scale(crack/2, 1.0f);

				img.crack->draw((int)blocks[i]->getx(), (int)blocks[i]->gety());

				//img.crack->set_scale(crack/2, 1.0f);
				img.crack->draw((int)blocks[i]->getx() + int(40 * (1 - crack/2)), (int)blocks[i]->gety());

			}
		}	

	}

	DB_ExitSubsection();
}

void testy::hitBlock(int myblock, CBall *myball){
	int scoreGain = 0;
	int blockx, blocky;

	blockx = (int) blocks[myblock]->getx();
	blocky = (int) blocks[myblock]->gety();

	if (realPhysics)
	{
		float vsq = myball->ballvx * myball->ballvx + myball->ballvy * myball->ballvy;
		float vratio;
		float newx = myball->ballvx, newy = myball->ballvy;

		vratio = .9f;

		//if (vsq < 5000)
		//	return;

		if (vratio < 0.9f) vratio = 0.9f;

		/*if (myball->ballvy * myball->ballvy < 100)
		{
			myball->maxv = paddleImbueV;
			return;
		}*/

		if (verticalhit)
		{
			newy = vratio * newy;

			if (abs(int(newy - myball->ballvy)) < 4) 
			{
				newy = abs(int(myball->ballvy)) - 4.0f;

				if (abs(int(newy + myball->ballvy)) < 
					abs(int(newy)) + abs(int(myball->ballvy))) newy = 0;
				if (myball->ballvy < 0) newy = -newy;

			}
		}
		else
		{
			newx = vratio * newx;


		}

	//	vratio = (newx * newx + newy * newy) / (myball->ballvx * myball->ballvx + myball->ballvy * myball->ballvy);



		if (newy * newy < 1000 && verticalhit) 
		{
			newx *= 0.95f;
		}

		myball->ballvx = newx;
		myball->ballvy = newy;
		myball->maxv = sqrtf(newx * newx + newy * newy - PE(myball));


		if (newy * newy < 3000 && verticalhit) 	return;
		if (newx * newx < 3000 && !verticalhit)  return;

	}

	// decrement the strength of the block
	blocks[myblock]->decreaseStr();

	// if we have the fireball and the block isn't cogglestone or steel, bust it up!
	if (fireball && blocks[myblock]->type == 'g') { blocks[myblock]->decreaseStr(blocks[myblock]->getStr()); }


	if (blocks[myblock]->getStr() == 0) 
	{
		dropBlockParts(blocks[myblock]->getx(), blocks[myblock]->gety(), blocks[myblock]->blockpart, myball->ballvx, myball->ballvy);
		blocksdestroyed++;
		


		
		if (blaster)
			scoreGain = 1;
		else 	
		{
			scoreGain = 11 - balls.size();

			if (fireball)
				scoreGain -= 5;

			if (paddlew > 110) 
				scoreGain -= 2;
			else if (paddlew < 90)
				scoreGain += 2;
			
			if (paddleImbueV > 510) scoreGain += 2;
			if (paddleImbueV < 500) scoreGain -= 2;

		}

		if (scoreGain < 1) scoreGain = 1;
			
		if (blocks[myblock]->type == 'i')
		{
			invincibleBlocks--;
			scoreGain = 0;
		}

		if(blocksdestroyed % blocksforitem == 0) 
		{
			dropPowerUp((int) blocks[myblock]->getx(), (int) blocks[myblock]->gety(), false);
		}
		else if(blocksdestroyed % blocksforpoints == 0) 
		{
			dropPowerUp((int) blocks[myblock]->getx(), (int) blocks[myblock]->gety(), true);
		}


		delete blocks[myblock];
        blocks.erase(blocks.begin() + myblock);

	}

	else
    {
		// check to see that the block is one we deserve points for
		if (blocks[myblock]->type != 'i')
		{
			scoreGain = 2;

			// adjust the amount of points gained based on the various
			// powerups in effect.
			if (paddleImbueV > 510) scoreGain += 1;
			if (paddleImbueV < 490) scoreGain -= 1;
			if (paddlew > 110) scoreGain -= 1;
			if (paddlew < 90) scoreGain += 1;
			if (blaster) scoreGain = 0;

			if (scoreGain < 0) scoreGain = 0;
		}
	
		// if the block didn't get destroyed, light it up :) and play sound
		dropFlash(blocks[myblock]->getx(), blocks[myblock]->gety());
		snd.bounce->play();
	
	}

	// woot!
	gainPoints(scoreGain, int(blockx + 20), int(blocky + 10));

}




void testy::dropBlockParts(float myx, float myy, CL_Sprite *myblock, float ballvx, float ballvy) {

	snd.shatter->play();

	if ((ballvx < 0) && verticalhit) {
		CBlockPart *pblock0= new CBlockPart((ballvx + ballvx * 0.5f), (float)-ballvy, myx, myy, myblock);
		CBlockPart *pblock1= new CBlockPart((float)ballvx, (float) -ballvy, myx + 20, myy, myblock);
		CBlockPart *pblock2= new CBlockPart((ballvx + ballvx * 0.5f), -ballvy + (-ballvy * 0.2f), myx, myy + 10, myblock);
		CBlockPart *pblock3= new CBlockPart((float)ballvx, -ballvy + (-ballvy * 0.2f), myx + 20, myy + 10, myblock);
		blockparts.push_back(pblock0);		
		blockparts.push_back(pblock1);
		blockparts.push_back(pblock2);
		blockparts.push_back(pblock3);
		return;
		}	

	if ((ballvx > 1) && verticalhit) {
		CBlockPart *pblock0= new CBlockPart((ballvx + ballvx * 0.5f),(float) -ballvy, myx+20, myy, myblock);
		CBlockPart *pblock1= new CBlockPart((float)ballvx,(float) -ballvy, myx, myy, myblock);
		CBlockPart *pblock2= new CBlockPart((ballvx + -ballvx * 0.5f), -ballvy + (-ballvy * 0.2f), myx+20, myy + 10, myblock);
		CBlockPart *pblock3= new CBlockPart((float)ballvx, -ballvy + (-ballvy * 0.2f), myx, myy + 10, myblock);
		blockparts.push_back(pblock0);		
		blockparts.push_back(pblock1);
		blockparts.push_back(pblock2);
		blockparts.push_back(pblock3);
		return;
		}	

	if ((ballvx < 0) && !verticalhit) {
		CBlockPart *pblock0= new CBlockPart((-ballvx + -ballvx * 0.5f), (float)ballvy, myx, myy, myblock);
		CBlockPart *pblock1= new CBlockPart((float)-ballvx, (float)ballvy, myx + 20, myy, myblock);
		CBlockPart *pblock2= new CBlockPart((-ballvx + -ballvx * 0.5f), ballvy + (ballvy * 0.2f), myx, myy + 10, myblock);
		CBlockPart *pblock3= new CBlockPart((float)-ballvx, ballvy + (ballvy * 0.2f), myx + 20, myy + 10, myblock);
		blockparts.push_back(pblock0);		
		blockparts.push_back(pblock1);
		blockparts.push_back(pblock2);
		blockparts.push_back(pblock3);
		return;
		}	

	if ((ballvx > 1) && !verticalhit) {
		CBlockPart *pblock0= new CBlockPart((-ballvx + -ballvx * 0.5f), (float)ballvy, myx+20, myy, myblock);
		CBlockPart *pblock1= new CBlockPart((float)-ballvx, (float)ballvy, myx, myy, myblock);
		CBlockPart *pblock2= new CBlockPart((-ballvx + -ballvx * 0.5f), ballvy + (ballvy * 0.2f), myx+20, myy + 10, myblock);
		CBlockPart *pblock3= new CBlockPart((float)-ballvx, ballvy + (ballvy * 0.2f), myx, myy + 10, myblock);
		blockparts.push_back(pblock0);		
		blockparts.push_back(pblock1);
		blockparts.push_back(pblock2);
		blockparts.push_back(pblock3);
		return;
		}	


	}

void testy::updateBlockParts() 
{
	DB_EnterSubsection("updateBlockParts");

	if (blockparts.size() > 0) 
	{
		unsigned int i;
		for(i=0; i < blockparts.size(); i++) 
		{
			if(!blockparts[i]->update(fps)) 
			{
				// delete it if the alpha is gone....
				deleteBlockPart(i);
				i--;

			}

		}	

	}
	
	DB_ExitSubsection();
}

void testy::deleteBlockPart(int myblockpart)
{
	delete blockparts[myblockpart];
    blockparts.erase(blockparts.begin() + myblockpart);
}

void testy::deleteAllBlockParts()
{
	while (blockparts.size() > 0)
		deleteBlockPart(0);
}

void testy::drawBlockParts() 
{
	DB_EnterSubsection("drawBlockParts");

	if (blockparts.size() > 0) 
	{
		unsigned int i;
		for(i=0; i < blockparts.size(); i++) 
		{
			blockparts[i]->block->set_alpha(blockparts[i]->alpha);
			blockparts[i]->block->set_angle(blockparts[i]->rotation);
			blockparts[i]->block->draw((int)blockparts[i]->x, (int)blockparts[i]->y);
		
		}	

	}

	DB_ExitSubsection();

}

void testy::dropFlash(float myx, float myy)
{
	CFlash *pFlash= new CFlash((int)myx,(int) myy);
	flashes.push_back(pFlash);
}

void testy::updateFlashes() 
{
	DB_EnterSubsection("updateFlashes");

	if (flashes.size() > 0) 
	{
		unsigned int i;
		for (i=0; i < flashes.size(); i++) 
		{
			if(!flashes[i]->update()) 
			{
				// if there's no alpha, delete it
				deleteFlash(i);
				i--;	

			}

		}

	}

	DB_ExitSubsection();
}

void testy::deleteFlash(int myflash) {
	delete flashes[myflash];
	flashes.erase(flashes.begin() + myflash);
	}

void testy::deleteAllFlashes()
{
	while (flashes.size() > 0)
		deleteFlash(0);
}

void testy::drawFlashes() 
{
	DB_EnterSubsection("drawFlashes");

	if(flashes.size() > 0) 
	{
		unsigned int i;
		for (i=0; i < flashes.size(); i++) 
		{
			img.flash->set_alpha(flashes[i]->alpha);
			img.flash->draw((int) flashes[i]->x, (int) flashes[i]->y);
		
		}

	}

	DB_ExitSubsection();
}

void testy::addBall() 
{
	CBall *myBall= new CBall;
	myBall->stickydifference= paddlew * 0.5f;
	balls.push_back(myBall);

	//perfectgame = false;
}

void testy::addBall(int offset) 
{
	CBall *myball= new CBall;
	myball->stickydifference= (float)offset;
	balls.push_back(myball);

	//perfectgame = false;
}


void testy::deleteBall(int myball) 
{
	//CBall *theball = balls[myball];
	delete balls[myball];
	balls.erase(balls.begin() + myball);
	
	//delete theball;
}

void testy::deleteAllBalls() {

	if (balls.size() > 0) {
		unsigned int i;
		for (i= 0; i < balls.size(); i++) {
			delete balls[i];
			}
		balls.clear();
		}
	}

void testy::powerUp(ePowerups effect)
{
	CPowerUp dummy(0, 0);

	dummy.setEffect(effect);

	powerUp(&dummy);
}

void testy::powerUp(CPowerUp *powerup, int extraPoints) 
{
	int scoreGain = extraPoints;
	char effect = powerup->getEffect();

	powerupcount ++;

	switch (effect)
	{
	case PTS100:
		scoreGain = 100;
		powerupcount --;

		break; 

	case PTS250:
		scoreGain = 250;
		powerupcount --;

		break; 

	case PTS500:
		scoreGain = 500;
		powerupcount --;

		break; 

	case PTS1000:
		scoreGain = 1000;
		powerupcount --;

		break; 

	case ONEUP:
		lives++;
		scoreGain = 50;
	
		break; 

	case STICKY:
	
		scoreGain = 50;
		stickypaddle = true;
	
		break; 

	case SUPERSTICKY:
		
		scoreGain = 50;
		supersticky = true;

		break;

	case MULTIBALL: 
	
		addBall(int(paddlew * 0.5f) + 0);
		addBall(int(paddlew * 0.5f) +10);
		addBall(int(paddlew * 0.5f) -10);
		addBall(int(paddlew * 0.5f) -20);
		scoreGain = 50;
	
		break; 

	case BLASTER: 
	
		blaster= true;
		scoreGain = 50;
		addBall();
		//perfectgame = false;

	
		break; 

	case FIREBALL: 
	
		fireball= true;
		scoreGain = 50;
	
		break;

	case FASTBALL:
	
		scoreGain = 50;
		
		if (balls.size() > 0) 
		{
			paddleImbueV = 700.0f;
			/*int i;
			for (i= 0; i < balls.size(); i++) {
				float angle= atan2(balls[i]->ballvy, balls[i]->ballvx);
				balls[i]->ballvx= maxv * cos(angle);
				balls[i]->ballvy= maxv * sin(angle);
				}*/
		}	

	
		break; 

	case SLOWBALL:

		scoreGain = 50;
		if (balls.size() > 0) 
		{
			paddleImbueV = 325.0f;
			/*int i;
			for (i= 0; i < balls.size(); i++) {
				float angle= atan2(balls[i]->ballvy, balls[i]->ballvx);
				balls[i]->ballvx= maxv * cos(angle);
				balls[i]->ballvy= maxv * sin(angle);
				}*/
		}		

	
		break;

	case NORMALSPEED:
	
		scoreGain = 50;
		if (balls.size() > 0) 
		{
			paddleImbueV = 500.0f;
			/*int i;
			for (i= 0; i < balls.size(); i++) {
				float angle= atan2(balls[i]->ballvy, balls[i]->ballvx);
				balls[i]->ballvx= maxv * cos(angle);
				balls[i]->ballvy= maxv * sin(angle);
				}*/
		}

		break;

	case LARGEPADDLE:

		scoreGain = 50;
		setPaddleSize(150);
	
		break; 

	case SMALLPADDLE: 
	
		scoreGain = 50;
		setPaddleSize(50);

		break;

	case REGULARPADDLE:
	
		scoreGain = 50;
		setPaddleSize(100);

		break;

	case CATCHBLUE:

		scoreGain = 50;
		catchblue = true;
		catchbluestart = CL_System::get_time();

		break;

	case CATCHRED:

		scoreGain = 50;
		catchred = true;
		catchredstart = CL_System::get_time();

		break;

	case RESET: 
	
		scoreGain = 50;

		// adjust the amount of points gained for whether we are
		// losing "good" or "bad" powerups
		if (blaster) scoreGain += 25;
		if (fireball) scoreGain += 25;
		if (paddlew < 100) scoreGain -= 25;
		if (paddlew > 100) scoreGain += 25;
		if (paddleImbueV > 510) scoreGain -= 25;
		if (paddleImbueV < 490) scoreGain += 25;
		if (stickypaddle) scoreGain += 25;
		if (supersticky) scoreGain += 25;
		if (catchblue) scoreGain += 25;
		if (catchred) scoreGain -= 25;

		setPaddleSize(100);
		catchblue = catchred = stickypaddle = supersticky = blaster = fireball = false;
		paddleImbueV = 500.0f;



		/*if (balls.size() > 0) 
		{
			int i;
			for (i= 0; i < balls.size(); i++) 
			{
				float angle= atan2(balls[i]->ballvy, balls[i]->ballvx);
				balls[i]->ballvx= maxv * cos(angle);
				balls[i]->ballvy= maxv * sin(angle);
			}
		}	*/
		
		break;

	case RANDOM:

		// this is worth 50 extra points, but we pass it to the actual powerup
		// so that the scorebyte that's displayed on the screen is the right
		// number.
		scoreGain = 0;

		// declare some local variables, so enclose in braces to 
		// avoid stupid warnings.
		{
			CPowerUpList list;
			CPowerUp *actual;

			// build the list of available powerups.
			BuildPowerUpList(list);
			list.removeEffect(RANDOM);

			// choose one of them
			list.AssignPowerup(&actual, 0, 0);

			// execute it
			powerUp(actual, 50);

			// change my icon:
			powerup->icon = actual->icon;
			
			// delete it.
			delete actual;
		}

		break;
	}

	// don't let the point gain drop below zero
	if (scoreGain < 0) scoreGain = 0;

	// gain the actual points
	gainPoints(scoreGain, int(powerup->x + 20), int(powerup->y + 20));

}

void testy::setPaddleSize(float fNewSize)
{
	float oldSize = paddlew;
	paddlew = fNewSize;

	if (balls.size() > 0) 
	{
		// now, adjust the positions of the balls, so none of them
		// are way off the edge of the paddle.
		unsigned int i;
		for (i= 0; i < balls.size(); i++) 
		{
			if (balls[i]->ballsticking)
			{
				// the formula is to keep the balls at the same ratio of distance
				// from the edge of the paddle to the size of the paddle
				float oldPos = balls[i]->stickydifference / oldSize;

				balls[i]->stickydifference = oldPos * paddlew;
			}

		}
	}

	// make sure the paddle does not go outside the edge.
	paddleCheckEdge();
}

void testy::BuildPowerUpList(CPowerUpList &pulist)
{
	// erase whatever's in the list now
	pulist.clear();

	// only list the powerups in the list that will actually have an effect
	if (paddlew < 110)					pulist.addEffect(LARGEPADDLE,		GetPUIcon(LARGEPADDLE),		5);
	if (!blaster)						pulist.addEffect(BLASTER,			GetPUIcon(BLASTER),			1);
	if (paddleImbueV < 510)				pulist.addEffect(FASTBALL,		GetPUIcon(FASTBALL),		5);
	if (!fireball)						pulist.addEffect(FIREBALL,		GetPUIcon(FIREBALL),		1);
	if (90 > paddlew || paddlew > 110)	pulist.addEffect(REGULARPADDLE,	GetPUIcon(REGULARPADDLE),	2);
	if (paddlew > 90)					pulist.addEffect(SMALLPADDLE,		GetPUIcon(SMALLPADDLE),		3);
	if (490 > paddleImbueV || paddleImbueV > 510)
										pulist.addEffect(NORMALSPEED,		GetPUIcon(NORMALSPEED),		3);
	if (paddleImbueV > 490)				pulist.addEffect(SLOWBALL,		GetPUIcon(SLOWBALL),		2);

	if (!stickypaddle)					pulist.addEffect(STICKY,			GetPUIcon(STICKY),			1);
	if (stickypaddle && !supersticky)	pulist.addEffect(SUPERSTICKY,		GetPUIcon(SUPERSTICKY),		2);
	
	if (!catchblue)						pulist.addEffect(CATCHBLUE,		GetPUIcon(CATCHBLUE),		1);
	if (!catchred)						pulist.addEffect(CATCHRED,		GetPUIcon(CATCHRED),		2);

	// these should always be available for dropping under any conditions.
	pulist.addEffect(RESET,		GetPUIcon(RESET),		3);
	pulist.addEffect(MULTIBALL,	GetPUIcon(MULTIBALL),	2);
	pulist.addEffect(ONEUP,		GetPUIcon(ONEUP),		1);
	pulist.addEffect(RANDOM,		GetPUIcon(RANDOM),		6);

	//if (
	
/*	if (set >= 0 && set <= 4)    { ppowerup->effect = 'f'; ppowerup->icon = img.pupaddlelarge; }
	if (set >= 5 && set <= 5)    { ppowerup->effect = 'b'; ppowerup->icon = img.publaster; }
	if (set >= 6 && set <= 10)   { ppowerup->effect = 'c'; ppowerup->icon = img.pufastball; }
	if (set >= 11 && set <= 11)  { ppowerup->effect = 'd'; ppowerup->icon = img.pufireball; }
	if (set >= 12 && set <= 13)  { ppowerup->effect = 'e'; ppowerup->icon = img.pumultiball; }
	if (set >= 14 && set <= 14)  { ppowerup->effect = 'a'; ppowerup->icon = img.pu1up; }
	if (set >= 15 && set <= 16)  { ppowerup->effect = 'g'; ppowerup->icon = img.pupaddleregular; }
	if (set >= 17 && set <= 19)  { ppowerup->effect = 'h'; ppowerup->icon = img.pupaddlesmall; }
	if (set >= 20 && set <= 22)  { ppowerup->effect = 'i'; ppowerup->icon = img.puregularspeed; }
	if (set >= 23 && set <= 24)  { ppowerup->effect = 'j'; ppowerup->icon = img.puslowball; }
	if (set >= 25 && set <= 25)  { ppowerup->effect = 'k'; ppowerup->icon = img.pusticky; }
	if (set >= 26 && set <= 28)  { ppowerup->effect = 'l'; ppowerup->icon = img.pureset; }
*/
}

CL_Sprite *testy::GetPUIcon(ePowerups effect)
{

	switch (effect)
	{
		case ONEUP:				return img.pu1up;
		case BLASTER:			return img.publaster;
		case FASTBALL:			return img.pufastball;
		case FIREBALL:			return img.pufireball;
		case MULTIBALL:			return img.pumultiball;
		case LARGEPADDLE:		return img.pupaddlelarge;
		case REGULARPADDLE:		return img.pupaddleregular;
		case SMALLPADDLE:		return img.pupaddlesmall;
		case NORMALSPEED:		return img.puregularspeed;
		case SLOWBALL:			return img.puslowball;
		case STICKY:			return img.pusticky;
		case RESET:				return img.pureset;
		case PTS100:			return img.pu100;
		case PTS250:			return img.pu250;
		case PTS500:			return img.pu500;
		case PTS1000:			return img.pu1000;
		case RANDOM:			return img.purandom;
		case CATCHBLUE:			return img.pucatchblue;
		case CATCHRED:			return img.pucatchred;
		case SUPERSTICKY:		return img.pusupersticky;

		default:				return NULL;
	}
}

void testy::dropScoreByte(int myx, int myy, int amount)
{
//	int width;
	string str;
	CL_Color clr;

	if (amount > 0)
	{
		str = "+" + itos(amount);
		clr = CL_Color(0, 255, 0, 255);
	}
	else if (amount < 0)
	{
		str = "-" + itos(-amount);
		clr = CL_Color(255, 0, 0, 255);
	}
	else // amount == 0
		return;
		
	img.font->set_scale(1.0f, 1.0f);

	//int pitch = displayFormat->get_pitch();
	int width = img.font->get_width(str);
	int height = img.font->get_height(str);


	myx -= width / 2;
	myy -= height / 2;

	//*surf = new CL_Surface(
	CScoreByte *sb = new CScoreByte(myx, myy, str, NULL /*surf*/, clr);

	scoreBytes.push_back(sb);

}

void testy::updateScoreBytes()
{
	for (unsigned int i = 0; i < scoreBytes.size(); i++)
	{
		scoreBytes[i]->update(float(fps));

		if (scoreBytes[i]->getAlpha() < 0.001f)
			deleteScoreByte(i);

	}
}

void testy::deleteScoreByte(int myscore)
{
	delete scoreBytes[myscore];
	scoreBytes.erase(scoreBytes.begin() + myscore);
}

void testy::deleteAllScoreBytes()
{
	while (scoreBytes.size() > 0)
		deleteScoreByte(0);
}

void testy::drawScoreBytes()
{
	if (scoreBytes.size() > 0)
	{
		for (unsigned int i = 0; i < scoreBytes.size(); i++)
		{
			CScoreByte *mybyte = scoreBytes[i];
			
			img.font->set_color(mybyte->getColor());
			img.font->set_alpha(mybyte->getAlpha());	
			img.font->draw(mybyte->getx(), mybyte->gety(), mybyte->getAmount());
				
		}
	}

}

void testy::dropPowerUp(int myx, int myy, bool pointsonly)
{
	CPowerUp *ppowerup;
	CPowerUpList pulist;
	int set;
	
	BuildPowerUpList(pulist);

	if (pointsonly) 
	{
		ppowerup = new CPowerUp((float)myx, (float)myy);

		set = ppowerup->start % 20;

		if (set >= 0 && set <= 10)  { ppowerup->setEffect(PTS100); ppowerup->icon = img.pu100; }
		if (set >= 11 && set <= 16)  { ppowerup->setEffect(PTS250); ppowerup->icon = img.pu250; }
		if (set >= 17 && set <= 18)  { ppowerup->setEffect(PTS500); ppowerup->icon = img.pu500; }
		if (set >= 19 && set <= 19)  { ppowerup->setEffect(PTS1000); ppowerup->icon = img.pu1000; }
	}
	else
	{
		pulist.AssignPowerup(&ppowerup, (float) myx, (float) myy);
	}


	powerups.push_back(ppowerup);

}

void testy::updatePowerUps() 
{
	if (powerups.size() > 0) 
	{
		unsigned int i;
		bool badPowerup;



		for(i=0; i < powerups.size(); i++) 
		{
			bool hitpaddle= true;
			badPowerup = false;

			CPowerUp *mypowerup = powerups[i];

			if(mypowerup->x > paddlex + paddlew) hitpaddle=false;
			if(mypowerup->y > paddley + paddleh) hitpaddle=false;
			if(mypowerup->x + 40 < paddlex) hitpaddle=false;
			if(mypowerup->y + 40 < paddley) hitpaddle=false;

			if(catchblue && mypowerup->isBlue())
				hitpaddle = true;
			if(catchred && mypowerup->isRed()) 
				hitpaddle = true;

			if(mypowerup->getEffect() == 'z') hitpaddle = false;

			if(dying)					hitpaddle = false;

			if(hitpaddle) 
			{
				powerUp(mypowerup);
				mypowerup->setEffect(PU_NONE);
				mypowerup->vx = paddleVelocity/10;

				snd.powerup->play();

			}
			if(!mypowerup->update(fps)) 
			{
				// delete it if the alpha is gone....
				badPowerup = true;

			}


			if (badPowerup)
			{
				deletePowerUp(i);
				i--;
			}
		}	
	}
}


void testy::deletePowerUp(int mypowerup){
	delete powerups[mypowerup];
    powerups.erase(powerups.begin() + mypowerup);
	}

void testy::deleteAllPowerUps() {

	if (powerups.size() > 0) {
		unsigned int i;
		for (i= 0; i < powerups.size(); i++) {
			delete powerups[i];
			}
		powerups.clear();
		}
	}

void testy::drawPowerUps() 
{
	DB_EnterSubsection("drawPowerUps");

	if (powerups.size() > 0) 
	{
		unsigned int i;
		for(i=0; i < powerups.size(); i++) 
		{
			CPowerUp *mypowerup = powerups[i];

			mypowerup->icon->set_alpha(mypowerup->a);
			mypowerup->icon->set_color(mypowerup->r,mypowerup->r,mypowerup->r, mypowerup->a);
			mypowerup->icon->set_scale(mypowerup->w, mypowerup->h);
			mypowerup->icon->draw((int)mypowerup->x, (int)mypowerup->y);

			if (mypowerup->oldeffect == RANDOM)
			{
				img.purandom->set_alpha(mypowerup->a);
				img.purandom->set_color(mypowerup->r,mypowerup->r,mypowerup->r, mypowerup->a);
				img.purandom->set_scale(mypowerup->h, mypowerup->h);
				img.purandom->draw((int)mypowerup->x, (int)mypowerup->extray);
			}
		}	

	}

	DB_ExitSubsection();
}

//-------------- TITLE SCREEN ===============================================================================

string testy::runTitle() {

	unsigned int levelstart;
	
	CL_SoundBuffer_Session session;
	session = snd.music1->prepare();
	session.set_looping(true);
	if (playmusic) session.play();
	lives=2;
	thescore= 0;
	img.arrow->set_color(1.0f, 1.0f, 1.0f, 1.0f);
	titlemode= " ";
	
	level = (int)ceil(rand() / 32768.0f * 35);

	initLevel();
	attractMode = true;
	attractvelocity = 0;

	hideTitle = false;

	lastMouseMove = CL_System::get_time();
	levelstart = lastMouseMove;
	
	beginninglevel = 1;
	beginningchanged = false;

	unsigned int lastframetime = CL_System::get_time();
	unsigned int frametime = CL_System::get_time();

	do 
	{
		
		DB_DefineNewFrameDescriptor();

		fps= myfps->get_fps();

		//fps = 1/(frametime - lastframetime);

		if(session.is_playing() && !playmusic) session.stop();
		if(!session.is_playing() && playmusic) session.play();
		
		updateLevel();
		updateTitle();

		draw();
		drawTitle();


		lastframetime = frametime;
		frametime = CL_System::get_time();

		if (frametime - lastMouseMove > 15000)
			hideTitle = true;

		// check to see if attract mode died or won.
		if (blocks.size() <= invincibleBlocks || balls.size() == 0 
			|| frametime - levelstart > 100000 || beginningchanged)
		{
			if (beginningchanged)
				level = beginninglevel;
			else
				level = (int)ceil(rand() / 32768.0f * 35);

			beginningchanged = false;

			initLevel();
			attractMode = true;
			attractvelocity = 0;

			levelstart = frametime;
		}

		if(titlemode == "quit") {
			if(session.is_playing()) session.stop();
			return "quit";
			}
		if(titlemode == "leveleditor")
		{
			level = beginninglevel;
			if(session.is_playing()) session.stop();
			return "editor";
		}
		if(titlemode == "startgame") 
		{
			attractMode = false;
			lives = 2;
			thescore = 0;
			level = beginninglevel;

			beginninglevel = 0;

			if(session.is_playing()) session.stop();
			return "level";
		}

		DB_EndFrame();

	} while(!CL_Keyboard::get_keycode(CL_KEY_ESCAPE)); 

	if(session.is_playing()) session.stop();
	return "quit";
	}

void testy::updateTitle() {
	if ( (fps > 0) ) {

// update bg scroll for fun... 
		bgy += bgspeed / (float)fps;
		if(bgy >= 300.0f) bgy = bgy - 300.0f;

		}
	}

void testy::drawTitle() {
	//CL_Display::clear(CL_Color(0, 0, 0, 255));

// draw the background tile
	/*img.bgtile->draw(0, -300 + (int)bgy);
	img.bgtile->draw(400, -300 + (int)bgy);
	img.bgtile->draw(0, 0 + (int)bgy);
	img.bgtile->draw(400, 0 + (int)bgy);
	img.bgtile->draw(0, 300 + (int)bgy);
	img.bgtile->draw(400, 300 + (int)bgy);
*/
// draw perimeter
	img.topborder->draw(0,0);
	img.leftborder->draw(0,0);
	img.rightborder->draw(735,0);
	img.topborder->draw(0,590);

	CL_Display::fill_rect(
		CL_Rect( 0, 0, 800, 600 ),
		CL_Color(0, 0, 0, 100)
		);

	// hidetitle becomes true after a period of mouse inactivity.
	if (!hideTitle)
	{
		CL_Display::fill_rect(
			CL_Rect( 95, 95, 800, 322 ),
			CL_Color(0, 0, 0, 100)
			);

		img.font->set_scale(2.0f, 2.0f);
		if(mousex > 100-20 && mousex < 500 && mousey > 100 && mousey < 130) {
			img.font->set_color(1.0f, 1.0f, 0.0f, 1.0f);
			}
		img.font->draw(100, 100, "[START THE GAME]");
		img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
		if(mousex > 100-20 && mousex < 500 && mousey > 130 && mousey < 160) {
			img.font->set_color(1.0f, 1.0f, 0.0f, 1.0f);
			}
		img.font->draw(100, 130, "[LEVEL EDITOR]");
		img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
		if(mousex > 100-20 && mousex < 500 && mousey > 160 && mousey < 190) {
			img.font->set_color(1.0f, 1.0f, 0.0f, 1.0f);
			}
		img.font->draw(100, 160, "[QUIT]");
		img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
		if(mousex > 100-20 && mousex < 500 && mousey > 190 && mousey < 220) {
			img.font->set_color(1.0f, 1.0f, 0.0f, 1.0f);
			}
		img.font->draw(100, 190, "[Full Screen / Windowed]");
		img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
		if(mousex > 100-20 && mousex < 500 && mousey > 220 && mousey < 250) {
			img.font->set_color(1.0f, 1.0f, 0.0f, 1.0f);
			}
		if (bgscroll) img.font->draw(100, 220, "[Background Scroll On]");
		if (!bgscroll) img.font->draw(100, 220, "[Background Scroll Off]");
		img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
		if(mousex > 100-20 && mousex < 500 && mousey > 250 && mousey < 280) {
			img.font->set_color(1.0f, 1.0f, 0.0f, 1.0f);
			}
		if (vsync) img.font->draw(100, 250, "[VSync On]");
		if (!vsync) img.font->draw(100, 250, "[VSync Off]");
		img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
		if(mousex > 100-20 && mousex < 500 && mousey > 280 && mousey < 310) {
			img.font->set_color(1.0f, 1.0f, 0.0f, 1.0f);
			}
		if (playmusic) img.font->draw(100, 280, "[Play Music]");
		if (!playmusic) img.font->draw(100, 280, "[Do not play Music]");
		img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);


			// draw high scores
		img.font->set_scale(2.0f, 2.0f);
		img.font->draw(500, 100, "High scores");
		img.font->set_scale(1.0f, 1.0f);


		for (unsigned int i = 0; i < highscores.size(); i++)
		{
			int drawy = 150 + 15 * i;

			img.font->draw(500, drawy, itos(i+1) + string(". "));
			img.font->draw(530, drawy, highscores[i]->name);
			img.font->draw(680, drawy, itos(highscores[i]->score));

		}

		img.bblogo->draw(40, 350);

		img.xlogo->update();
		img.xlogo->draw(640, 350);

	}
	else
	{
		img.font->set_scale(1.0f, 1.0f);

		if (CL_System::get_time() % 100 < 50)
			img.font->set_color(CL_Color(255, 120, 120));
		else
			img.font->set_color(CL_Color(255, 255, 255));

		img.font->draw(330, 50, "Move mouse for title screen.");
	}
	
	img.font->set_scale(1.0f, 1.0f);
	img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);

	img.font->draw(100, 500, "Starting on level " + itos(beginninglevel));

	img.font->draw(100, 540, "Ball: Buster, by Patrick Avella (C) 2004");
	img.font->draw(100, 555, "Ball: Buster eXtreme modifications made by Erik Ylvisaker (C) 2004");
	img.font->draw(100, 570, "Because Breaking Stuff is Fun");	
	img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);

	img.font->set_scale(1.0f, 1.0f);
	img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);

	

	if (!hideTitle)
	{
		// draw cursor
		img.arrow->draw(mousex,mousey);
	}


	CL_Display::flip(vsync);
	//CL_System::sleep(0);
	CL_System::keep_alive();
}







//------------ EDITOR
void testy::loadEditor() {
	clearBlocks();
    ifstream myfile;
	char charbuf[20];
	itoa(level, charbuf, 10);
	string mystr= charbuf;
	mystr= DATADIR "lvls/" + mystr + ".lvl";
	myfile.open(mystr.c_str());

	if (myfile.bad())
		return;

	mystr = "_________________";

	int j= 0;
	for (j = 0; j < 20; j++) {
		getline(myfile, mystr);
		int i;
		for (i= 0; i < 17; i++){
			if((char)mystr[i] != '_') { 
				addBlock((60 + (40 * i)), 60 + (20 * j), (char)mystr[i]);
				}
			}
		}
	myfile.close();
	}




void testy::drawEditor() {
	CL_Display::clear(CL_Color(0, 0, 0, 255));

// draw the background tile
	img.bgtile->draw(0, -300 + (int)bgy);
	img.bgtile->draw(400, -300 + (int)bgy);
	img.bgtile->draw(0, 0 + (int)bgy);
	img.bgtile->draw(400, 0 + (int)bgy);
	img.bgtile->draw(0, 300 + (int)bgy);
	img.bgtile->draw(400, 300 + (int)bgy);

// draw perimeter
	img.topborder->draw(0,0);
	img.leftborder->draw(0,0);
	img.rightborder->draw(735,0);

// draw blocks and update their animations...
	img.rblock->update();
	img.rblockpart->update();
	img.gblock->update();
	img.gblockpart->update();
	img.bblock->update();
	img.bblockpart->update();
	img.yblock->update();
	img.yblockpart->update();
	img.tblock->update();
	img.tblockpart->update();
	img.wblock->update();
	img.wblockpart->update();

	drawBlocks();

// we draw the flash right on top of the block
	drawFlashes();

// we'll draw the pieces of block last, so it looks cool....

// draw whatever text
	char charbuf[64];
	itoa(level, charbuf, 10);
//	itoa(score, charbuf, 10);

	string message= charbuf;
	message= "EDITING LEVEL: " + message;
	img.font->set_color(0.0f, 0.0f, 0.0f, 1.0f);	
	img.font->draw(11,585, message);
	img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
	img.font->draw(13,583, message);


	if(mousex > 13-20 && mousex < 80 && mousey > 523 && mousey < 538) {
		img.font->set_color(1.0f, 1.0f, 0.0f, 1.0f);
		}
	img.font->draw(13, 523, "[TITLE SCREEN]");
	img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
	if(mousex > 13-20 && mousex < 80 && mousey > 538 && mousey < 553) {
		img.font->set_color(1.0f, 1.0f, 0.0f, 1.0f);
		}
	img.font->draw(13, 538, "[SAVE LEVEL]");
	img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
	if(mousex > 13-20 && mousex < 80 && mousey > 553 && mousey < 568) {
		img.font->set_color(1.0f, 1.0f, 0.0f, 1.0f);
		}
	img.font->draw(13, 553, "[UP LEVEL]");
	img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
	if(mousex > 13-20 && mousex < 80 && mousey > 568 && mousey < 583) {
		img.font->set_color(1.0f, 1.0f, 0.0f, 1.0f);
		}
	img.font->draw(13, 568, "[DOWN LEVEL]");
	img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);


	if(mousex > 40 && mousex < 80+40*16 && mousey > 60 && mousey < 60+20*20) {
		CL_Display::fill_rect(
			CL_Rect(mousex - (mousex%40) + 20, mousey - (mousey%20), mousex - (mousex%40) + 60, mousey - (mousey%20) + 20),
			CL_Color(100, 255, 100, 100)
			);
		}

	img.font->draw(200, 540, "Choose your Brush:");
	int mytempi= 200;
	img.rblock->draw(mytempi, 560);	mytempi+=40;
	img.gblock->draw(mytempi, 560);	mytempi+=40;
	img.bblock->draw(mytempi, 560);	mytempi+=40;
	img.yblock->draw(mytempi, 560);	mytempi+=40;
	img.tblock->draw(mytempi, 560);	mytempi+=40;
	img.wblock->draw(mytempi, 560);	mytempi+=40;
	img.cblock->draw(mytempi, 560);	mytempi+=40;
	img.sblock->draw(mytempi, 560);	mytempi+=40;
	img.pureset->draw(mytempi, 560);	mytempi+=40;

// draw cursor
	img.arrow->draw(mousex,mousey);


	CL_Display::flip(vsync);
	CL_System::sleep(0);
	CL_System::keep_alive();
	}





string testy::runEditor() {

	bgspeed= bgscroll ? 50.0f : 0.0f;

	clearBlocks();
	
	loadEditor();

	//fireball= true;

	brush= 'r';
	img.arrow->set_color(1.0f, 0.0f, 0.0f, 1.0f);

	while(!CL_Keyboard::get_keycode(CL_KEY_ESCAPE)) {
		fps= myfps->get_fps();

		updateEditor();
		drawEditor();
		if(editormode == "reload") { editormode= " "; return "editor"; }
		if(editormode == "titlescreen") { editormode= " "; return "title"; }
		}
	return "title";
	}


void testy::updateEditor() {
	if ( (fps > 0) ) {

		handleEditorClicks();

// update the block fragments and the brick flashes
		updateFlashes();

// update bg scroll for fun... 
		bgy += bgspeed / (float)fps;
		if(bgy >= 300.0f) bgy = bgy - 300.0f;

		}
	}

CBlock* testy::findBlockAt(int myx, int myy) {

	if (blocks.size() > 0) {
		unsigned int i;
		for (i= 0; i < blocks.size(); i++) {
			if(blocks[i]->getx() == myx && blocks[i]->gety() == myy)
			return blocks[i];
			}
		}
	return NULL;
	}

void testy::deleteBlockAt(int myx, int myy) {

	if (blocks.size() > 0) {
		unsigned int i;
		for (i= 0; i < blocks.size(); i++) {
			if(blocks[i]->getx() == myx && blocks[i]->gety() == myy) {
				delete blocks[i];
				blocks.erase(blocks.begin() + i);
				return;
				}
			}
		}
	}

void testy::handleEditorClicks() {

	if(mousedown) {
		int mytempi= 180;
		if(mousex > mytempi && mousex < mytempi+40 && mousey > 560 && mousey < 580) {
			img.arrow->set_color(1.0f, 0.0f, 0.0f, 1.0f);
			brush= 'r';
			}
		mytempi+= 40;
		if(mousex > mytempi && mousex < mytempi+40 && mousey > 560 && mousey < 580) {
			img.arrow->set_color(0.0f, 1.0f, 0.0f, 1.0f);
			brush= 'g';
			}
		mytempi+= 40;
		if(mousex > mytempi && mousex < mytempi+40 && mousey > 560 && mousey < 580) {
			img.arrow->set_color(0.0f, 0.0f, 1.0f, 1.0f);
			brush= 'b';
			}
		mytempi+= 40;
		if(mousex > mytempi && mousex < mytempi+40 && mousey > 560 && mousey < 580) {
			img.arrow->set_color(1.0f, 1.0f, 0.0f, 1.0f);
			brush= 'y';
			}
		mytempi+= 40;
		if(mousex > mytempi && mousex < mytempi+40 && mousey > 560 && mousey < 580) {
			img.arrow->set_color(0.0f, 1.0f, 1.0f, 1.0f);
			brush= 't';
			}
		mytempi+= 40;
		if(mousex > mytempi && mousex < mytempi+40 && mousey > 560 && mousey < 580) {
			img.arrow->set_color(1.0f, 1.0f, 1.0f, 1.0f);
			brush= 'w';
			}
		mytempi+= 40;
		if(mousex > mytempi && mousex < mytempi+40 && mousey > 560 && mousey < 580) {
			img.arrow->set_color(0.85f, 0.5f, 0.5f, 1.0f);
			brush= 'c';
			}
		mytempi+= 40;
		if(mousex > mytempi && mousex < mytempi+40 && mousey > 560 && mousey < 580) {
			img.arrow->set_color(0.5f, 0.5f, 0.5f, 1.0f);
			brush= 's';
			}
		mytempi+= 40;
		if(mousex > mytempi && mousex < mytempi+40 && mousey > 560 && mousey < 600) {
			img.arrow->set_color(1.0f, 0.3f, 0.3f, 0.5f);
			brush= 'x';
			}
		}
	if(mousex > 40 && mousex < 80+40*16 && mousey > 60 && mousey < 60+20*20
		&& !findBlockAt(mousex - (mousex%40) + 20, mousey - (mousey%20)) && 
		mousedown && brush != 'x' ) 
	{
		dropFlash(mousex - (mousex%40) + 20.0f, float( mousey - (mousey%20)));
		addBlock(mousex - (mousex%40) + 20, mousey - (mousey%20), brush);
	}
	if(mousex > 40 && mousex < 80+40*16 && mousey > 60 && mousey < 60+20*20
		&& findBlockAt(mousex - (mousex%40) + 20, mousey - (mousey%20)) && 
		mousedown && brush == 'x') 
	{
		dropFlash(float(mousex - (mousex%40) + 20), float(mousey - (mousey%20)));
		deleteBlockAt(mousex - (mousex%40) + 20, mousey - (mousey%20));
//		addBlock(mousex - (mousex%40) + 20, mousey - (mousey%20), brush);
		}

	}




void testy::saveEditor() {
    ofstream myfile;
	char charbuf[20];
	itoa(level, charbuf, 10);
	string mystr= charbuf;
	mystr= DATADIR "lvls/" + mystr + ".lvl";
	myfile.open(mystr.c_str());

	int j, i;
	CBlock *pblock;
	for(j=0; j < 20; j++) {
		for(i=0; i < 17; i++) {
			pblock= findBlockAt(60 + (40 * i), 60 + (20 * j));
			if(pblock) myfile << pblock->color;
			if(!pblock) myfile << "_";
			}
		myfile << "\n";
		}

    myfile.close();
	snd.ching->play();

	}

void testy::splash() {

	unsigned int start= CL_System::get_time();

	CL_Display::clear(CL_Color(255, 255, 255, 255));

	img.palogo->draw(335, 100);

	img.font->set_color(0.0f, 0.0f, 0.0f, 1.0f);
	img.font->set_scale(1.0f, 1.0f);
	img.font->draw(200, 250, "Ball: Buster, Copyright 2004 Patrick Avella");
	img.font->draw(200, 265, "Game Programming: Patrick Avella (patrickavella.com)");
	img.font->draw(200, 280, "eXtreme Version Programming: Erik Ylvisaker");
	img.font->draw(200, 295, "Game Art: Patrick Avella (patrickavella.com)");
	img.font->draw(200, 310, "ClanLib SDK Programming: The ClanLib dev team (clanlib.org)");
	img.font->draw(200, 325, "Background Music: Partners in Rhyme (musicloops.com)");
	img.font->draw(200, 340, "Sound Effects: A1 Free Sound Effects (a1freesoundeffects.com)");

	CL_Display::flip(vsync);

	while((start + 3000 > CL_System::get_time()) && (getkeypress() == 0)) {
		CL_System::sleep(0);
		CL_System::keep_alive();
		}

	img.font->set_color(1.0f, 1.0f, 1.0f, 1.0f);
	}





void testy::loadhighscores()
{
	ifstream hs;
	char buffer[100];

#ifdef __unix__
	char *home = getenv("HOME");
	if (home)
	{
		snprintf(buffer, sizeof(buffer), "%s/.ballbuster.hs", home);
		hs.open(buffer);
	}
	else
#endif
	hs.open("highscores");
	
	if (!hs.is_open())
	{
		char *defaulths[] = {
		  "Skel,50000",
		  "Kanato,40000",
		  "Ball: Buster X,30000",
		  "Alison,20000",
		  "Melvin,10000",
		  "Your Mom,5000" };
		for (int i = 0; i < 6; i++)
		{
			CHighscore *myscore = new CHighscore(defaulths[i]);
			highscores.push_back(myscore);
		}
	}
	else
	while (!hs.eof())
	{
		CHighscore *myscore;

		memset(buffer, 0, sizeof(buffer));

		hs.getline(buffer, 100);

		try
		{
			if (buffer[0] != 0)
			{
				myscore = new CHighscore(buffer);

				highscores.push_back(myscore);
			}
		}
		catch (char*)
		{
			// failed to create high scores.  That's ok, when the file is saved, it
			// will be removed.
		}
			

	}

}

void testy::savehighscores()
{
	ofstream hs;


#ifdef __unix__
	char *home = getenv("HOME");
	char buffer[100];
	if (home)
	{
		snprintf(buffer, sizeof(buffer), "%s/.ballbuster.hs", home);
		hs.open(buffer);
	}
	else
#endif
	hs.open("highscores");


	for (unsigned int i = 0; i < highscores.size(); i++)
	{
		hs << highscores[i]->toString() << endl;
	}

}

int testy::getkeypress()
{
	int temp = hitkey;
	hitkey = 0;
	return temp;
}

void testy::keyPress(const CL_InputEvent &key)
{
	hitkey    = key.id;
	hitstring = key.str;

	// check to see if the physics engine should be turned on/off
	// physics engine is disabled.
	/*if (hitkey == CL_KEY_PRIOR)
		realPhysics = true;
	else if (hitkey == CL_KEY_NEXT)	
		realPhysics = false;
	*/

	// check to see if we should change the starting level
	if (beginninglevel > 0)
	{
		if (hitkey == CL_KEY_UP)
		{
			beginninglevel++;
			beginningchanged = true;
		}
		else if (hitkey == CL_KEY_DOWN && beginninglevel > 1)
		{
			beginninglevel--;
			beginningchanged = true;
		}

	}
	
	if (key.id == CL_KEY_P || key.id == CL_KEY_PAUSE && !transitionout)
	{
		paused = !paused;
	}
}

unsigned long testy::getScore()
{
	return thescore;
}

void testy::gainPoints(unsigned long pts, int x, int y)
{
	if (freezeScore)
		return;

	thescore += pts;


	if (x > 0)
	{
		dropScoreByte(x, y, pts);
	}


}


