/*****************************************************************************
	Ball: Buster
	Copyright (C) 2004 PatrickAvella

    This file is part of Ball: Buster.

    Ball: Buster is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ball: Buster is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Ball: Buster; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*******************************************************************************
**  Ball: Buster uses the ClanLib sdk, it can be found at http://clanlib.org
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
*******************************************************************************/


#include "headers.h"
#include "CImage.h"
#include "block.h"
#include "blockpart.h"
#include "flash.h"
#include "ball.h"
#include "powerup.h"
#include "CSound.h"
#include "BBUtility.h"
#include "ScoreByte.h"
#include "Highscores.h"




class testy : public CL_ClanApplication {
public:
	testy();
	~testy();

	virtual int main(int, char **);

	void run();

	void mouseMove(const CL_InputEvent &mouse);
	void mousePress(const CL_InputEvent &key);
	void mouseRelease(const CL_InputEvent &key);
	void keyPress(const CL_InputEvent &key);
	

private:
	CL_PixelBuffer *displayFormat;

// the variables here are REALLY sloppy, sorry......... :P
	string gamemode;
	bool attractMode;
	int attractvelocity;
	bool hideTitle;
	unsigned int lastMouseMove;
	int beginninglevel;
	bool beginningchanged;

	//char powerup;
	bool stickypaddle;
	bool supersticky;
	bool catchblue, catchred;
	bool /*ballsticking,*/ fireball, blaster, stageover, transitionout, died, dying;
	bool bgscroll, playmusic;
	//int ballstickstart, stickydifference;
	unsigned int catchbluestart, catchredstart;
	bool paused;

	int vsync;

	int transitionspeed, transdelay;
	unsigned int transstart, deathstart;
	int lives;
	int blocksforitem;    // these are for the system of dropping items....
	int blocksforpoints;
	int blocksdestroyed;
	int transcount;
	
	unsigned int invincibleBlocks;

	//unsigned long int score;
	unsigned long int thescore;
	bool freezeScore;
	int level;

	int powerupcount;
	int ballslost;

	string debuginfo;

	bool verticalhit;
	bool debugger;

	float bgy, bgspeed;

	CImage img;
	CSound snd;

	float paddlex, paddley, paddlew, paddleh, paddlerot, paddlealpha;
	float paddleVelocity;
	float paddleImbueV;
/*	float ballx, bally, ballw, ballh;
	float ballvx, ballvy; */

	//float maxv; // velocity
	
	vector<CBlock*> blocks;
	vector<CBlockPart*> blockparts;
	vector<CFlash*> flashes;
	vector<CBall*> balls;
	vector<CPowerUp*> powerups;
	vector<CScoreByte*> scoreBytes;


	vector<CHighscore*>	highscores;


	CL_FramerateCounter *myfps;
	int fps;


	CL_Slot mousemotion;
	CL_Slot mousepress, mouserelease;
	CL_Slot keypress;

	int hitkey;
	string hitstring;
	int getkeypress();

	string runTitle();
	void updateTitle();
	void drawTitle();

	string runLevel();
	void endLevel(unsigned int levelTime);
	void initLevel();
	void loadLevel();
	void updateLevel();
	void draw();

	int powerupTop, powerupLeft;
	void drawPowerupInEffect(CL_Sprite *image, int secondsleft = -1);

	void endgame();

	bool paddleCheckEdge();
	void setPaddleSize(float fNewSize);

	CL_SoundBuffer_Session playRandomSong();

	void addBlock(int, int, char color);
	void clearBlocks();

	void drawBlocks();
	void hitBlock(int, CBall*);

	void dropBlockParts(float myx, float myy, CL_Sprite *myblock, float ballvx, float ballvy);
	void updateBlockParts();
	void deleteBlockPart(int myblockpart);
	void drawBlockParts();
	void deleteAllBlockParts();


	void dropFlash(float myx, float myy);
	void updateFlashes();
	void deleteFlash(int myflash);
	void drawFlashes();
	void deleteAllFlashes();

	void dropScoreByte(int myx, int myy, int amount);
	void updateScoreBytes();
	void deleteScoreByte(int myscore);
	void drawScoreBytes();
	void deleteAllScoreBytes();


	void addBall();
	void addBall(int offset); // add's a ball with an offset against the paddle
	void deleteBall(int myball);
	void deleteAllBalls();

	void SetGravityVelocity (CBall *myball);
	float PE(CBall *myball);

	void powerUp(CPowerUp *powerup, int extraPoints = 0);
	void powerUp(ePowerups effect); 

	void BuildPowerUpList(CPowerUpList &list);
	void dropPowerUp(int myx, int myy, bool pointsonly);
	void updatePowerUps();
	void deletePowerUp(int mypowerup);
	void deleteAllPowerUps();
	void drawPowerUps();	
	CL_Sprite *GetPUIcon(ePowerups effect);

	unsigned long getScore();
	void gainPoints(unsigned long pts, int myx = -1, int myy = -1);

//	void deleteAllPowerUps();

//-----------------------

	int mousex, mousey;
	bool mousedown;

//=======================
	string titlemode;
//----------------------

	bool realPhysics;

	char brush;
	string editormode;
	bool fullscreen;

	string runEditor();
	void loadEditor();
	void saveEditor();
	void updateEditor();
	void drawEditor();
	CBlock* findBlockAt(int myx, int myy);
	void deleteBlockAt(int myx, int myy);
	void handleEditorClicks();



	void splash();


	void loadhighscores();
	void savehighscores();

};


