/*****************************************************************************
	Ball: Buster
	Copyright (C) 2004 PatrickAvella

    This file is part of Ball: Buster.

    Ball: Buster is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ball: Buster is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Ball: Buster; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*******************************************************************************
**  Ball: Buster uses the ClanLib sdk, it can be found at http://clanlib.org
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
*******************************************************************************/

#include "powerup.h"


CPowerUp::CPowerUp(float myx, float myy)
{
	this->start= CL_System::get_time();
	this->delay= 100;
	this->w = this->h = 1.0f;
	this->r = this->g = this->b = 1.0f;
	this->a = 1.0f;

	this->x= myx; 
	this->y= myy;
	this->extray = y;

	this->vx= 0.0f;
	this->vy= 100.0f;
	
	this->isred = this->isblue = false;
}

void CPowerUp::setEffect(ePowerups meffect)
{
	oldeffect = effect;
	effect = meffect;
	
	/*
		ONEUP = 'a',
	BLASTER = 'b',
	FASTBALL = 'c',
	FIREBALL = 'd',
	MULTIBALL = 'e',
	LARGEPADDLE = 'f',
	REGULARPADDLE = 'g',
	SMALLPADDLE = 'h',
	NORMALSPEED = 'i',
	SLOWBALL = 'j',
	STICKY = 'k',
	RESET = 'l',
	PTS100 = 'm',
	PTS250 = 'n',
	PTS500 = 'o',
	PTS1000 = 'p',
	RANDOM = 'r',
	CATCHBLUE = 's',
	CATCHRED = 't',
	*/
	switch (effect)
	{
		case RANDOM:

			isred = isblue = true;
			break;

		case FASTBALL:
		case SMALLPADDLE:
		case CATCHRED:
		case BLASTER:
		
			isred = true;
			break;

		case MULTIBALL:
		case LARGEPADDLE:
		case SLOWBALL:
		case STICKY:
		case PTS100:
		case PTS250:
		case PTS500:
		case CATCHBLUE:

			isblue = true;
			break;
	}

}

char CPowerUp::getEffect()
{
	return effect;
}

bool CPowerUp::update(int myfps) 
{
	if (this->effect != 'z') 
	{ 
		// we'll set the effect to z when the player attains the power up
		this->y += vy / (float)myfps;
		this->x += vx / (float)myfps;
		this->extray = this->y;

		if ((unsigned)this->delay + start < CL_System::get_time())
		{
			this->vy+= 20.0f;
			this->start= CL_System::get_time();
		}
		if(this->y > 600) return false;
		return true;
	}

	if ((unsigned)this->delay + start < CL_System::get_time()) 
	{
		this->w+= 0.06f;
		this->x-= 0.03f * icon->get_width();
		this->h-= 0.03f;
//		this->r-= 0.03f;
		this->a-= 0.03f;
		this->y -= vy / (float)myfps;
		this->x += vx / (float)myfps;
		this->extray -= (vy * 1.5f) / (float)myfps;


		if(this->a <= 0.0f) return false;
	}

	return true;
}


CPowerUpList::CPowerUpList()
{
	total = 0;
}

CPowerUpList::~CPowerUpList()
{
	clear();
}

void CPowerUpList::addEffect(ePowerups effect, CL_Sprite *icon, int weight)
{
	PUData *pudata = new PUData(effect, icon, weight);

	total += weight;

	data.push_back(pudata);

}

void CPowerUpList::clear()
{
	while (data.size() > 0)
	{
		delete data[0];
		data.erase(data.begin());
	}

	total = 0;
}

void CPowerUpList::AssignPowerup(CPowerUp **powerup, float x, float y)
{
	int start = (signed) CL_System::get_time();
	PUData *sel;

	start %= total;

	for (unsigned int i = 0; i < data.size(); i++)
	{
		start -= data[i]->weight;

		if (start < 0)
		{
			sel = data[i];
			break;
		}

	}

	
	*powerup = new CPowerUp(x,y);
	(*powerup)->setEffect(sel->effect);
	(*powerup)->icon = sel->icon;
	
}


void CPowerUpList::removeEffect(ePowerups effect)
{
	for (unsigned int i = 0; i < data.size(); i++)
	{
		if (data[i]->effect == effect)
		{
			total -= data[i]->weight;

			delete data[i];
			data.erase(data.begin() + i);
		}

	}
}
